# gozerplugs/timebomb.py
#
# Gozerbot Timebomb plugin by Clone 2009 
# Idea not so loosely based on jotham.read@gmail.com's eggdrop timebomb 

""" do the timebomb dance. """

__copyright__ = 'BSD'
__author__ = 'clone at magtar.org'

from gozerbot.generic import getwho
from gozerbot.commands import cmnds
from gozerbot.plughelp import plughelp
from gozerbot.persist.persist import PlugPersist
from gozerbot.aliases import aliasset

from time import sleep, time
from random import randint, shuffle

plughelp.add('timebomb', 'blow your buddies to smithereens !timebomb <victim>')
plughelp.add('cut', 'try to defuse a bomb placed with !timebomb by cutting a wire i.e. !cut blue')

# define plugpersist outside localscope, you only want to initiate it once.
bomb = PlugPersist('bomb')
bomb.data = []

def timebomb(bot, ievent):
    # check if we have ops
    if ievent.channel not in bot.state['opchan']:
        bot.action(ievent.channel, "bends over and farts in %s's general direction." % ievent.nick)
        return
    # check if we are already running a bomb
    if bomb.data:
        bot.action(ievent.channel ,"points at the bulge in %s's pants." % bomb.data[0])
        return
    try:
        userhost = getwho(bot, ievent.args[0])
    except IndexError:
        ievent.reply('timebomb requires victim, see !help timebomb.')
        return
    # check if the victim userhost exists on this channel
    if not userhost:
         ievent.reply('no %s here.' % ievent.args[0])
         return
    else:
        user = ievent.args[0]
    # if bot gets targeted, switch target to caller
    if ievent.args[0].lower() == bot.nick.lower():
         userhost = ievent.ruserhost 
         user = ievent.nick
    # define wires 
    wires = ['blue','black','red','green','purple','white','silver'];
    # determine number of wires and pick random colors
    shuffle(wires)
    mywires = wires[0:randint(2,len(wires)-1)]
    counter = 18 + 2 * len(mywires) + randint(1,12)
    # determine time to mark instance
    instancetime = time()
    # plant bomb: (name to kick, which wires to choose from, which wire disarms, userhost)
    bomb.data = [user, mywires, mywires[randint(0,len(mywires)-1)], userhost, instancetime]
    ievent.reply('%s places a bomb in %s\'s pants, the timer reads %s seconds. You see the wires :%s' % (ievent.nick, user, counter, str(mywires)))
    # wait for timer to expire
    sleep(counter)
    
    # check if persist data still exists (no cut event) and kick if so.
    if bomb.data:
        # data from different instance, dont cut
        if not bomb.data[-1] == instancetime:
           return
        else: 
            #kick victim
            bot.sendraw('KICK %s %s :%s' % (ievent.channel, bomb.data[0], 'B000000M!'))
            #ievent.reply('user: %s, userhost: %s' % (bomb.data[0], bomb.data[3]))
            bomb.data = []

def cut(bot, ievent):
    # check if there is a timebomb running
    if bomb.data:
        # right userhost?
        if bomb.data[3] == ievent.ruserhost:
            # right wire?
            if ievent.args[0] == bomb.data[2]:
                 bomb.data=[]
                 ievent.reply('%s has defused the bomb.' % ievent.nick)
            else:
                bot.sendraw('KICK %s %s :%s' % (ievent.channel, bomb.data[0], 'snip...B000000M!'))
                bomb.data=[]
            
cmnds.add('timebomb', timebomb, 'USER', threaded=True)
aliasset('tb', 'timebomb')
cmnds.add('cut', cut, 'USER', threaded=True)



# Description: Performs a sed-like substitution on the last message by the 
#              calling user
# Author: John Hampton <pacopablo@pacopablo.com>
# Website: http://pacopablo.com
# License: BSD

__author__ = 'John Hampton <pacopablo@pacopablo.com>'
__license__ = "BSD"

# Standard Library Imports
import os
import time
import re

# Third Party Imports

# Local Imports
from gozerbot.callbacks import callbacks
from gozerbot.commands import cmnds
from gozerbot.datadir import datadir
from gozerbot.persist.pdod import Pdod
from gozerbot.persist.persistconfig import PersistConfig
from gozerbot.plughelp import plughelp
from gozerbot.examples import examples
from gozerbot.redispatcher import rebefore

plughelp.add('sed', 'Perform substitution on last message spoken')

cfg = PersistConfig()
cfg.define('cmd_req', 0)
sed_expression = r'^s([/|#.:;])(.*?)\1(.*?)\1?([gi]*)$'
sedre = re.compile(sed_expression)

class LastLine(Pdod):
    def __init__(self):
        self.datadir = os.path.join(datadir, 'plugs', 'sed')
        Pdod.__init__(self, os.path.join(self.datadir, 'sed.data'))
        if not self.data:
            self.data = {}

    def handle_sed(self, bot, ievent):
        """ Perform substitution """
        channel = ievent.channel.lower()
        nick = ievent.nick.lower()
        try:
            (delim, broke, fix, flags) = ievent.groups
        except ValueError:
            ievent.missing('<delim><broke><delim><fix><delim>')
            return
        try:
            source = self.data[channel][nick]
            if 'g' in flags:
                count = 0
            else:
                count = 1
            if 'i' in flags:
                broke = '(?i)'+broke
            new_text = re.sub(broke, fix, source, count)

            if source != new_text:
                ievent.reply("%s meant: %s" % (nick, new_text))
                return
        except KeyError:
            bot.say(nick, 'I wasn\'t listening to you.  Try saying something first.')
            return
        except Exception, ex:
            bot.say(nick, 'Error processing regex: %s' % str(ex))
            return

    def precb(self, bot, ievent):
        if not ievent.usercmnd:
            return 1

    def privmsgcb(self, bot, ievent):
        channel = ievent.channel.lower()
        nick = ievent.nick.lower()
        regex = sedre.match(ievent.origtxt)
        if not cfg.get('cmd_req') and regex:
            try:
                (delim, broke, fix, flags) = regex.groups()
            except ValueError:
                return
            try:
                source = self.data[channel][nick]
                if 'g' in flags:
                    count = 0
                else:
                    count = 1
                if 'i' in flags:
                    broke = '(?i)'+broke
                new_text = re.sub(broke, fix, source, count)
                if source != new_text:
                    ievent.reply("%s meant: %s" % (nick, new_text))
                    return

            except KeyError:
                return
            except Exception, ex:
                ievent.reply('Error processing regex: %s' % str(ex))
        self.data.setdefault(channel, {})
        self.data[channel][nick] = ievent.origtxt

lastline = None

def handle_sed(bot, ievent):
    global lastline
    lastline.handle_sed(bot, ievent)

def init():
    global lastline
    lastline = LastLine()
    callbacks.add('PRIVMSG', lastline.privmsgcb, lastline.precb)
    callbacks.add('Message', lastline.privmsgcb, lastline.precb)
    rebefore.add(10, sed_expression, handle_sed, 'USER', allowqueue=False)
    examples.add('s', 'Perform substitution on last message spoken.', 's/foo/bar/')
    return 1

def shutdown():
    if lastline:
        lastline.save()
    



# plugs/botsnack.py
#
#

""" eat it """

__copyright__ = 'this file is in the public domain'

from gozerbot.utils.generic import convertpickle
from gozerbot.datadir import datadir
from gozerbot.commands import cmnds
from gozerbot.examples import examples
from gozerbot.persist.persist import PlugPersist
from gozerbot.plughelp import plughelp
import random, os

plughelp.add('botsnack', 'give the bot a snack')

## UPGRADE PART

def upgrade():
    convertpickle(datadir + os.sep + 'old' + os.sep + 'botsnacklist', \
datadir + os.sep  + "plugs" + os.sep + "botsnack" + os.sep + 'botsnacklist')

## END UPGRADE PART

bsl = PlugPersist('botsnacklist')

if not bsl.data:
    bsl = PlugPersist('botsnacklist')
    if not bsl.data:
        bsl.data = []

def handle_botsnack(bot, ievent):
    """ botsnack .. give botsnack reply """
    if bsl.data:
        result = random.choice(bsl.data)
        result = result.replace('<nick>', ievent.nick)
        result = result.replace('<host>', ievent.userhost)
        ievent.reply(result)
    else:
        ievent.reply('smikkel ;]')

cmnds.add('botsnack', handle_botsnack, ['USER', 'CLOUD'])
examples.add('botsnack', 'give the bot a snack ;] .. botsnack responses \
can be added with botsnack-add', 'botsnack')

def handle_addbotsnack(bot, ievent):
    """ botsnack-add <reply> .. add botsnack reply """
    if not ievent.rest:
        ievent.missing('<what>')
        return
    # append and save
    bsl.data.append(ievent.rest)
    bsl.save()
    ievent.reply('botsnack added')

cmnds.add('botsnack-add', handle_addbotsnack, 'OPER', allowqueue=False)
examples.add('botsnack-add', 'botsnack-add <what> .. add a botsnack \
response, <nick> can be used to show nick of user giving the command', \
'botsnack-add thnx <nick> ;]')

def handle_listbotsnack(bot, ievent):
    """ botscnack-list .. list botsnack replies """
    ievent.reply(str(bsl.data))

cmnds.add('botsnack-list', handle_listbotsnack, 'OPER')
examples.add('botsnack-list', 'show list of botsnack replies', 'botsnack-list')

def handle_delbotsnack(bot, ievent):
    """ botsnack-del <reply> .. delete botsnack reply """
    if not ievent.rest:
        ievent.missing('<txt>')
        return
    try:
        bsl.data.remove(ievent.rest)
        bsl.save()
    except ValueError:
        ievent.reply('i have no %s in botsnacklist' % ievent.rest)
        return
    ievent.reply('botsnack %s removed' % ievent.rest)

cmnds.add('botsnack-del', handle_delbotsnack, 'OPER')
examples.add('botsnack-del', 'delete entry from botsnack list', 'botsnack-del \
thnx <nick> ;]')



# gozerplugs/mash.py
#
#

from gozerbot.generic import geturl, striphtml
from gozerbot.commands import cmnds
from gozerbot.examples import examples
from gozerbot.plughelp import plughelp
from simplejson import loads

plughelp.add('mash', 'query searchmash')

baseurl = 'http://www.searchmash.com/results/'

def handle_mash(bot, ievent):
    if not ievent.rest:
        ievent.missing('<what>')
        return
    data = geturl(baseurl + '+'.join(ievent.rest.split()))
    try:
        results = loads(data)
    except ValueError:
        ievent.reply("can't make results of %s" % data)
        return 
    res = []
    for result in results['results']:
        res.append('%s: - <%s>' % (striphtml(result['title']), result['url']))
    ievent.reply('results for %s: ' % ievent.rest, res, dot=' || ')

cmnds.add('mash', handle_mash, 'USER')
examples.add('mash', 'query searchmash', 'mash gozerbot')



# plugs/markov.py
#
#

"""

Markov Talk for Gozerbot

The Chain:
    (predictate) -> [list of possible words]

TODO:
    - Propabilities
    - Start searching for full sentence, not just the first ORDER_K words 
      of a sentence
"""

__copyright__ = 'this file is in the public domain'
__author__ =  'Bas van Oostveen'
__coauthor__ = 'Bart Thate <bart@gozerbot.org>'
__gendocfirst__ = ['markov-enable', ]
__gendoclast__ = ['markov-disable', ]
__depend__ = ['log', ]

from gozerbot.datadir import datadir
from gozerbot.generic import rlog, geturl, striphtml, jsonstring
from gozerbot.persist.persist import PlugPersist
from gozerbot.commands import cmnds
from gozerbot.examples import examples
from gozerbot.callbacks import callbacks, jcallbacks
from gozerbot.plughelp import plughelp
from gozerbot.plugins import plugins
from gozerbot.threads.thr import start_new_thread
from gozerbot.utils.statdict import Statdict
from gozerbot.utils.limlist import Limlist
from os.path import join as _j
import time, re, random, types

from gozerbot.persist.persistconfig import PersistConfig

plughelp.add('markov', 'Gozerbot speaking madness')

cfg = PersistConfig()
cfg.define('enable', [])
cfg.define('command', 1)
cfg.define('onjoin', [])

def enabled(botname, channel):
    if jsonstring([botname, channel]) in cfg['enable']:
        return True

# Markers (is Marker the correct name for this?)
class Marker: pass
class BeginMarker(Marker): pass
class EndMarker(Marker): pass
class NickMarker(Marker): pass

# Tokens
TOKEN = Marker()
TOKEN_BEGIN = BeginMarker()
TOKEN_END = EndMarker()
TOKEN_NICK = NickMarker()

# Order-k, use predictate [-k:] = [word,word,]
# if ORDER_K==1: { ('eggs'):['with','spam',], 'with': ['bacon','green',] }
# if ORDER_K==2: { ('eat','eggs'):['with',TOKEN,), ('eggs','with'): ['bacon',] }
# ...
# Logical setting is often 2 or 3
ORDER_K = 2

# Maximum generation cycles
MAXGEN = 500

markovlearn = PlugPersist('markovlearn', [])
markovwords = {}
markovwordi = []
markovchains = {}

cfg.define('loud', 1)

def init():
    """ init plugin """
    # check if enabled
    if not cfg.get('enable'):
        return 1
    # if so register callbacks
    callbacks.add("PRIVMSG", cb_markovtalk, cb_markovtalk_test, threaded=True)
    callbacks.add('JOIN', cb_markovjoin, threaded=True)
    jcallbacks.add('Message', cb_markovtalk, cb_jmarkovtalk_test, \
threaded=True)
    # learn log or url on startup
    start_new_thread(markovtrain, (markovlearn.data,))
    return 1

def size():
    """ return size of markov chains """
    return len(markovchains)

def markovtrain(l):
    """ train items in list """
    time.sleep(1)
    for i in l:
        if i.startswith('http://'):
            start_new_thread(markovlearnurl, (i,))
        else:
            start_new_thread(markovlearnlog, (i,))
    return 1
	
def iscommand(bot, ievent):
    """ check to see if ievent is a command """
    if not ievent.txt:
        return 0
    try:
        cc = bot.channels[ievent.channel]['cc']
    except (TypeError, KeyError):
        cc = None
    txt = ""
    if cc and ievent.txt[0] == cc:
        txt = ievent.txt[1:]
    if ievent.txt.startswith(bot.nick + ':') or \
ievent.txt.startswith(bot.nick + ','):
        txt = ievent.txt[len(bot.nick)+1:]
    oldtxt = ievent.txt
    ievent.txt = txt
    result = plugins.woulddispatch(bot, ievent)
    ievent.txt = oldtxt
    return result

def cb_markovjoin(bot, ievent):
    """ callback to run on JOIN """
    # check if its we who are joining
    nick = ievent.nick.lower()
    if nick in bot.splitted:
        return
    if nick == bot.nick.lower():
        return
    # check if (bot.name, ievent.channel) is in onjoin list if so respond
    try:
        onjoin = cfg.get('onjoin')
    except KeyError:
        onjoin = None
    if type(onjoin) != types.ListType:
        return
    if jsonstring([bot.name, ievent.channel]) in onjoin:
        txt = getreply(bot, ievent, ievent.nick + ':')
        if txt:
            ievent.reply('%s: %s' % (ievent.nick, txt))
            
def cb_markovtalk_test(bot, ievent):
    """ callback precondition """
    if not ievent.usercmnd:
        return 1

def cb_jmarkovtalk_test(bot, ievent):
    """ callback precondition """
    if not ievent.usercmnd:
        return 1

def cb_markovtalk(bot, ievent):
    """ learn from everything that is being spoken to the bot """
    # check for relay
    if ievent.txt.count('[%s]' % bot.nick) > 0:
        return
    # strip
    txt = strip_txt(bot, ievent.txt)
    # markovtalk_learn
    if enabled(bot.name, ievent.channel):
        markovtalk_learn(txt)
    # if command is set in config then we don't respond in callback
    elif not cfg.get('loud'):
        return 
    itxt = ievent.txt.lower()
    # check is bot.nick is in ievent.txt if so give response
    botnick = bot.nick.lower()
    #responsenicks = (botnick, botnick+":", botnick+",")
    if botnick in ievent.txt or cfg.get('loud') and ievent.msg: 
        # reply when called 
        result = getreply(bot, ievent, txt)
	# dont reply if answer is going to be the same as question
        if not result:
            return
        if result.lower() == txt.lower():
            return
        ievent.reply(result)

# re to strip first word of logline
txtre = re.compile('^\S+ ')

def markovlearnlog(chan):
    """ learn a log """
    from gozerplugs.log import logs    
    logmap = logs.getmmap(chan)
    if not logmap:
        rlog(10, 'markov', "can't get logfile of %s" % chan)
        return
    # process lines in mmaped file
    lines = 0
    rlog(10, 'markov', 'learning %s log' % chan)
    while 1:
        if lines % 10 == 0:
            time.sleep(0.001)
        line = logmap.readline()
        if not line:
            break
        try:
            items = line.strip().split(',', 4)
            if items[3] == 'bot@bot':
                continue
            if items[4].startswith('CMND:'):
                continue
        except:
            continue
        try:
            txt = re.sub(txtre, '', items[4])
            markovtalk_learn(txt + '\n')
        except IndexError:
            continue
        lines += 1
    logmap.close()
    rlog(10, 'markov', 'learning %s log done' % chan)
    return lines

def markovlearnurl(url):
    """ learn an url """
    lines = 0
    rlog(10, 'markov', 'learning %s' % url)
    try:
        f = geturl(url)
        for line in f.split('\n'):
            line = striphtml(line)
            if lines % 10 == 0:
                time.sleep(0.01)
            line = line.strip()
            if not line:
                continue
            markovtalk_learn(line)
            lines += 1
    except Exception, e:
        rlog(10, 'markov', str(e))
    rlog(10, 'markov', 'learning %s done' % url)
    return lines

def strip_txt(bot, txt):
    """ strip bot nick and addressing """
    # TODO: strip other nicks, preferably replacing them with something like 
    # TOKEN_NICK
    txt = txt.replace("%s," % bot.nick, "")
    txt = txt.replace("%s:" % bot.nick, "")
    txt = txt.replace("%s" % bot.nick, "")
    return txt.strip()

def msg_to_array(msg):
    """ convert string to lowercased items in list """
    return [word.strip().lower() for word in msg.strip().split()]

def mw(w):
    if not w in markovwords:
        wi = len(markovwordi)
        markovwordi.append(w)
        markovwords[w] = wi
        return wi
    return markovwords[w]

def o2i(order):
    return tuple(mw(w) for w in order)

def i2o(iorder):
    return tuple(markovwordi[i] for i in iorder)

def markovtalk_learn(text_line):
    """ this is the function were a text line gets learned """
    text_line = msg_to_array(text_line)
    length = len(text_line)
    order = [TOKEN, ] * ORDER_K
    for i in range(length-1):
        order.insert(0, text_line[i])
        order = order[:ORDER_K]
        next_word = text_line[i+1]
        key = markovchains.setdefault(o2i(order), [])
        if not next_word in key:
            key.append(mw(next_word))

def getreply(bot, ievent, text_line):
    """ get 20 replies and choose the largest one """
    if not text_line:
        return ""
    text_line = msg_to_array(text_line)
    wordsizes = {}
    maxsize = 0
    for i in text_line:
        wordsizes[len(i)] = i
        if len(i) > maxsize:
            maxsize = len(i)
    results = []
    keywords = ['is', ]
    max = maxsize
    for i in range(7):
        p = ['', ]
        try:
            p[0] = wordsizes[max]
        except KeyError:
            p[0] = random.choice(text_line)
        if len(p) > 2:
            p.append(random.choice(text_line))
        #random.shuffle(p)
        if len(p) < 2:
            p.append('is')
        line = getline(' '.join(p))
        if line and line not in results:
            results.append(line)
        else:
            max -= 1
            if max < 1:
                max = maxsize
    if not results:
        return ""
    #highest = []
    #high = 0
    #for i in results:
    #    if len(i) > high:
    #        highest.insert(0, i)
    #        high = len(i)
    #for i in results:
    #    if wordsizes[maxsize] in i:
    #            return i
    #res = random.choice(highest)
    res = []
    for result in results[:4]:
        if len(result.split()) > 1:
            res.append(result.capitalize())
    return '. '.join(res)

def getline(text_line):
    """ get line from markovvhains """
    text_line = msg_to_array(text_line)
    order = Limlist(ORDER_K)
    for i in range(ORDER_K):
        order.append(TOKEN)
    teller = 0
    for i in text_line[:ORDER_K-1]:
        order[teller] = i
        teller += 1
    output = ""
    for i in range(MAXGEN):
        try:
            successorList = i2o(markovchains[o2i(order)])
        except KeyError:
            continue
        #print successorList
        keyword = successorList[0]
        word = random.choice(successorList)
        if not word:
            break
        if word not in output:
            output = output + " "  + word
        order.insert(0, word)
        order = order[:ORDER_K]
    return output.strip()
    
def handle_markovsize(bot, ievent):
    """ markov-size .. returns size of markovchains """
    ievent.reply("I know %s phrases" % str(len(markovchains.keys())))

cmnds.add('markov-size', handle_markovsize, 'OPER')
examples.add('markov-size', 'size of markovchains', 'markov-size')

def handle_markovlearn(bot, ievent):
    """ command to let the bot learn a log or an url .. learned data 
        is not persisted """
    try:
        item = ievent.args[0]
    except IndexError:
        ievent.reply('<channel>|<url>')
        return
    if item.startswith('http://'):
        nrlines = markovlearnurl(item)
        ievent.reply('learned %s lines' % nrlines)
        return
    from gozerplugs.log import logs
    if item in logs.loglist:
        ievent.reply('learning log file %s' % item)
        nrlines = markovlearnlog(item)
        ievent.reply('learned %s lines' % nrlines)
        return
    else:
        ievent.reply('logging is not enabled in %s' % item)

cmnds.add('markov-learn', handle_markovlearn, 'OPER', threaded=True)
examples.add('markov-learn', 'learn a logfile or learn an url', \
'1) markov-learn #dunkbots 2) markov-learn http://gozerbot.org')
 
def handle_markovlearnadd(bot, ievent):
    """ add log or url to be learned at startup or reload """
    try:
        item = ievent.args[0]
    except IndexError:
        ievent.missing('<channel>|<url>')
        return
    if item in markovlearn.data:
        ievent.reply('%s is already in learnlist' % item)
        return
    markovlearn.data.append(item)
    markovlearn.save()
    handle_markovlearn(bot, ievent)
    ievent.reply('done')

cmnds.add('markov-learnadd', handle_markovlearnadd, 'OPER')
examples.add('markov-learnadd', 'add channel or url to permanent learning .. \
this will learn the item on startup', '1) markov-learnadd #dunkbots 2) \
markov-learnadd http://gozerbot.org')

def handle_markovlearnlist(bot, ievent):
    """ show the learnlist """
    ievent.reply(str(markovlearn.data))

cmnds.add('markov-learnlist', handle_markovlearnlist, 'OPER')
examples.add('markov-learnlist', 'show items in learnlist', \
'markov-learnlist')

def handle_markovlearndel(bot, ievent):
    """ remove item from learnlist """
    try:
        item = ievent.args[0]
    except IndexError:
        ievent.missing('<channel>|<url>')
        return
    if item not in markovlearn.data:
        ievent.reply('%s is not in learnlist' % item)
        return
    markovlearn.data.remove(item)
    markovlearn.save()
    ievent.reply('done')

cmnds.add('markov-learndel', handle_markovlearndel, 'OPER')
examples.add('markov-learndel', 'remove item from learnlist', \
'1) markov-learndel #dunkbots 2) markov-learndel http://gozerbot.org')

def handle_markov(bot, ievent):
    """ this is the command to make the bot reply a markov response """
    if not enabled(bot.name, ievent.channel):
        ievent.reply('markov is not enabled in %s' % ievent.channel)
        return
    if not ievent.rest:
        ievent.missing('<txt>')
        return
    result = getreply(bot, ievent, strip_txt(bot, ievent.rest))
    if result:
        ievent.reply(result)

cmnds.add('markov', handle_markov, ['USER', 'WEB', 'CLOUD'])
examples.add('markov', 'ask for markov response', 'markov nice weather')

def handle_markovonjoinadd(bot, ievent):
    """ add channel to onjoin list """
    try:
        channel = ievent.args[0]
    except IndexError:
        channel = ievent.channel
    if (bot.name, channel) in cfg.get('onjoin'):
        ievent.reply('%s already in onjoin list' % channel)
        return
    cfg.get('onjoin').append((bot.name, channel))
    cfg.save()
    ievent.reply('%s added' % channel)

cmnds.add('markov-onjoinadd', handle_markovonjoinadd, 'OPER')
examples.add('markov-onjoinadd', 'add channel to onjoin config', \
'1) markov-onjoinadd 2) markov-onjoinadd #dunkbots')

def handle_markovonjoinremove(bot, ievent):
    """ remove channel from onjoin list """
    try:
        channel = ievent.args[0]
    except IndexError:
        channel = ievent.channel
    try:
        cfg.get('onjoin').remove((bot.name, channel))
    except ValueError:
        ievent.reply("%s not in onjoin list" % channel)
        return
    cfg.save()
    ievent.reply('%s removed' % channel)

cmnds.add('markov-onjoinremove', handle_markovonjoinremove, 'OPER')
examples.add('markov-onjoinremove', 'remove channel from onjoin config', \
'1) markov-onjoinremove 2) markov-onjoinremove #dunkbots')

def handle_markovenable(bot, ievent):
    """ enable markov in a channel .. learn the log of that channel """
    try:
        channel = ievent.args[0]
    except IndexError:
        channel = ievent.channel
    if not enabled(bot.name, channel):
        cfg.get('enable').append(jsonstring([bot.name, channel]))
    else:
        ievent.reply('%s is already enabled' % channel)
        return
    cfg.save()
    markovlearn.data.append(channel)
    markovlearn.save()
    plugins.reload('gozerplugs', 'markov')
    ievent.reply('%s enabled' % channel)

cmnds.add('markov-enable', handle_markovenable, 'OPER')
examples.add('markov-enable', 'enable markov learning in [<channel>]', '1) \
markov-enable 2) markov-enable #dunkbots')

def handle_markovdisable(bot, ievent):
    """ disable markov in a channel """
    try:
        channel = ievent.args[0]
    except IndexError:
        channel = ievent.channel
    if enabled(bot.name, channel):
        cfg.get('enable').remove(jsonstring([bot.name, channel]))
    else:
        ievent.reply('%s is not enabled' % channel)
        return
    cfg.save()
    try:
        markovlearn.data.remove(channel)
        markovlearn.save()
    except ValueError:
        pass
    plugins.reload('gozerplugs', 'markov')
    ievent.reply('%s disabled' % channel)

cmnds.add('markov-disable', handle_markovdisable, 'OPER')
examples.add('markov-disable', 'disable markov learning in [<channel>]', '1) \
markov-disable 2) markov-disable #dunkbots')



# plugins/gcalc.py
# encoding: utf-8

__copyright__ = 'This file is in the public domain'

from gozerbot.commands import cmnds
from gozerbot.examples import examples
from gozerbot.plughelp import plughelp
import urllib2

plughelp.add('gcalc', 'use the google calculator')

def handle_gcalc(bot, ievent):
    if len(ievent.args) > 0:
        expr = " ".join(ievent.args).replace("+", "%2B").replace(" ", "+")
    else:
        ievent.missing('Missing an expression')
        return
    
    req = urllib2.Request("http://www.google.com/search?q=%s" % expr, 
                          None, 
                          {'User-agent': 'Mozilla/5.0 (X11; U; Linux i686; en-US; rv:1.8.1.11) Gecko/20071204 BonEcho/2.0.0.11'})
    data = urllib2.urlopen(req).read()
    
    if "<img src=/images/calc_img.gif width=40 height=30 alt=\"\">" not in data:
        ievent.reply('Your expression can\'t be evaluated by the google calculator')
    else:
        ans = data.split("<img src=/images/calc_img.gif width=40 height=30 alt=\"\">")[1].split("<b>")[1].split("</b>")[0]
        ievent.reply(ans.replace('<font size=-2> </font>', '').replace('&#215;', '*').replace('<sup>', '**').replace('</sup>', ''))

    return
    
cmnds.add('gcalc', handle_gcalc, 'USER')
examples.add('gcalc', 'Calculate an expression using the google calculator', 'gcalc 1 + 1')



# plugs/echo.py
#
#

""" simple echo command """

__copyright__ = 'this file is in the public domain'

from gozerbot.commands import cmnds
from gozerbot.examples import examples
from gozerbot.plughelp import plughelp

plughelp.add('echo', 'simply write back what you tell it')

def handle_echo(bot, ievent):
    """ say back what is being said """
    ievent.reply(ievent.rest)

cmnds.add('echo', handle_echo, 'USER')
examples.add('echo', "echo Hello World!", 'echo')




# plugs/shop.py
#
#

__copyright__ = 'this file is in the public domain'
__gendocfirst__ = ['shop', ]

from gozerbot.utils.generic import convertpickle
from gozerbot.generic import getwho, jsonstring
from gozerbot.users import users
from gozerbot.commands import cmnds
from gozerbot.examples import examples
from gozerbot.datadir import datadir
from gozerbot.persist.pdol import Pdol
from gozerbot.plughelp import plughelp
import os

plughelp.add('shop', ' maintain shopping lists per user')

## UPGRADE PART

def upgrade():
    convertpickle(datadir + os.sep + 'old' + os.sep + 'shops',  datadir + os.sep + 'plugs' + \
os.sep + 'shop' + os.sep + 'shops')

## END UPGRADE PART

shops = Pdol(datadir + os.sep + 'plugs' + os.sep + 'shop' + os.sep + 'shops')

## END UPGRADE PART

def size():
    """ return number of shops entries """
    return len(shops.data)

def sayshop(bot, ievent, shoplist):
    """ output shoplist """
    if not shoplist:
        ievent.reply('nothing to shop ;]')
        return
    result = []
    teller = 0
    for i in shoplist:
        result.append('%s) %s' % (teller, i))
        teller += 1
    ievent.reply("shoplist: ", result)

def handle_shop(bot, ievent):
    """ shop [<item>] .. show shop list or add <item> """
    if len(ievent.args) != 0:
        handle_shop2(bot, ievent)
        return
    shop = []
    username = users.getname(ievent.userhost)
    shop = shops[username]
    sayshop(bot, ievent, shop)

cmnds.add('shop', handle_shop, 'USER')

def handle_shop2(bot, ievent):
    """ add items to shop list """
    if not ievent.rest:
        ievent.missing('<shopitem>')
        return
    else:
        what = ievent.rest
    username = users.getname(ievent.userhost)
    shops[username] = what
    shops.save()
    ievent.reply('shop item added')

examples.add('shop', 'shop [<item>] .. show shop items or add a shop item', \
'1) shop 2) shop bread')

def handle_shopchan(bot, ievent):
    """ shop-chan [<item>] .. show channel shop list or add <item> """
    if len(ievent.args) != 0:
        handle_shopchan2(bot, ievent)
        return
    shop = shops[jsonstring((bot.name, ievent.channel))]
    sayshop(bot, ievent, shop)

cmnds.add('shop-chan', handle_shopchan, 'USER')
examples.add('shop-chan', 'shop-chan [<item>] .. show channel shop list or \
add <item>', '1) shop-chan 2) shop-chan milk')

def handle_shopchan2(bot, ievent):
    """ add items to shop list """
    if not ievent.rest:
        ievent.missing('<shopitem>')
        return
    else:
        what = ievent.rest
    shops[jsonstring((bot.name, ievent.channel))] = what
    shops.save()
    ievent.reply('shop item added')

def handle_addshop(bot, ievent):
    """ shop-add <username> <item> .. add items to shop list of <username>"""
    if len(ievent.args) < 2:
        ievent.missing('<username> <item>')
        return
    else:
        who = ievent.args[0]
        what = ' '.join(ievent.args[1:])
    userhost = getwho(bot, who)
    if not userhost:
        ievent.reply("can't find userhost of %s" % who)
        return
    # get username of use giviing the command
    username = users.getname(ievent.userhost)
    # get username of person we want to knwo the shop list of
    whoname = users.getname(userhost)
    if not whoname:
        ievent.reply("can't find user for %s" % userhost)
        return
    if users.permitted(userhost, username, 'shop'):
        shops[whoname] = what
        shops.save()
        ievent.reply('shop item added')
    else:
        ievent.reply("%s does not share shopping list with %s" % (who, \
username))

cmnds.add('shop-add', handle_addshop, 'USER')
examples.add('shop-add', 'shop-add <username> <item> .. add shop item of \
someone else', '1) shop-add test milk')

def handle_getshop(bot, ievent):
    """ shop-get <name> .. get items of someone elses shop list """
    if not ievent.rest:
        ievent.missing('<username>')
        return
    who = ievent.rest
    userhost = getwho(bot, who)
    if not userhost:
        ievent.reply("can't find userhost of %s" % who)
        return
    username = users.getname(ievent.userhost)
    whoname = users.getname(userhost)
    if not whoname:
        ievent.reply("can't find user for %s" % userhost)
        return
    if users.permitted(userhost, username, 'shop'):
        shop = shops[whoname]
        sayshop(bot, ievent, shop)
    else:
        ievent.reply("%s does not share shopping list with %s" % (who, \
username))

cmnds.add('shop-get', handle_getshop, ['USER', 'WEB'])
examples.add('shop-get', 'shop-get <name> .. get shop items of someone \
else', '1) shop-get test')

def handle_delshop(bot, ievent):
    """ shop-del <username> <listofnrs> .. delete items of someone elses \
        shop list """
    if len(ievent.args) < 2:
        ievent.missing('<username> <listofnrs>')
        return
    else:
        who = ievent.args[0]
    try:
        nrs = []
        for i in ievent.args[1:]:
            nrs.append(int(i))
    except ValueError:
        ievent.reply('%s is not an integer' % i)
        return
    userhost = getwho(bot, who)
    if not userhost:
        ievent.reply("can't find userhost of %s" % who)
        return
    username = users.getname(ievent.userhost)
    whoname = users.getname(userhost)
    if not whoname:
        ievent.reply("can't find user for %s" % userhost)
        return
    if users.permitted(userhost, username, 'shop'):
        try:
            shop = shops[whoname]
        except KeyError:
            ievent.reply('nothing to shop ;]')
            return
        nrs.sort()
        nrs.reverse()
        for i in range(len(shop)-1, -1 , -1):
            if i in nrs:
                try:
                    del shop[i]
                except IndexError:
                    pass
        shops.save()
        ievent.reply('shop item deleted')
    else:
        ievent.reply("%s does not share shopping list with %s" % (who, \
username))

cmnds.add('shop-del', handle_delshop, 'USER')
examples.add('shop-del', 'shop-del <username> <listofnrs> .. delete shop \
item of someone else', '1) shop-del test 2 4 5')

def handle_got(bot, ievent):
    """ got <listofnrs> .. remove items from shoplist """
    if len(ievent.args) == 0:
        ievent.missing('<list of nrs>')
        return
    try:
        nrs = []
        for i in ievent.args:
            nrs.append(int(i))
    except ValueError:
        ievent.reply('%s is not an integer' % i)
        return
    username = users.getname(ievent.userhost)
    try:
        shop = shops[username]
    except KeyError:
        ievent.reply('nothing to shop ;]')
        return
    if not shop:
        ievent.reply("nothing to shop ;]")
        return
    nrs.sort()
    nrs.reverse()
    teller = 0
    for i in range(len(shop)-1, -1 , -1):
        if i in nrs:
            try:
                del shop[i]
                teller += 1
            except IndexError:
                pass
    shops.save()
    ievent.reply('%s shop item(s) deleted' % teller)

cmnds.add('got', handle_got, 'USER')
examples.add('got', 'got <listofnrs> .. remove item <listofnrs> from shop \
list','1) got 3 2) got 1 6 8')

def handle_gotchan(bot, ievent):
    """ got-chan <listofnrs> .. remove items from channel shoplist """
    if len(ievent.args) == 0:
        ievent.missing('<list of nrs>')
        return
    try:
        nrs = []
        for i in ievent.args:
            nrs.append(int(i))
    except ValueError:
        ievent.reply('%s is not an integer' % i)
        return
    try:
        shop = shops[jsonstring((bot.name, ievent.channel))]
    except KeyError:
        ievent.reply("nothing to shop")
        return
    if not shop:
        ievent.reply("nothing to shop")
        return
    nrs.sort()
    nrs.reverse()
    teller = 0
    for i in range(len(shop)-1, -1 , -1):
        if i in nrs:
            try:
                del shop[i]
                teller += 1
            except IndexError:
                pass
    shops.save()
    ievent.reply('%s shop item(s) deleted' % teller)

cmnds.add('got-chan', handle_gotchan, 'USER')
examples.add('got-chan', 'got-chan <listofnrs> .. remove listofnrs from \
channel shop list', '1) got-chan 3 2) got-chan 1 6 8')



# gozerplugs/buzz.py
#
#

__author__ = 'unknown'

from gozerbot.commands import cmnds
from gozerbot.examples import examples
from gozerbot.plughelp import plughelp

plughelp.add('buzz', 'create a buzz word')

from random import randint

verb = ["implement","utilize","integrate","streamline","optimize",
	"evolve","transform","embrace","enable","orchestrate","leverage",
	"reinvent","aggregate","architect","enhance","incentivize","morph",
	"empower","envisioneer","monetize","harness","facilitate","seize",
	"disintermediate","synergize","strategize","deploy","brand","grow",
	"brand","grow","target","syndicate","synthesize","deliver","mesh",
	"incubate","engage","maximize","benchmark","expedite","reintermediate", 
	"expedite","reintermediate","whiteboard","visualize","repurpose",
	"innovate","scale","unleash","drive","extend","engineer",
	"revolutionize","generate","exploit","transition","e-enable","iterate",
	"cultivate","matrix","productize","redefine","recontextualize"]

adj = ["clicks-and-mortar","value-added","vertical","proactive",
	"robust","revolutionary","scalable","leading-edge","innovative",
	"intuitive","strategic","e-business","mission-critical","sticky",
	"one-to-one","24/7","end-to-end","global","B2B","B2C","granular",
	"frictionless","virtual","viral","dynamic","24/365","best-of-breed",
	"killer","magnetic","bleeding-edge","web-enabled","interactive",
	"dot-com","sexy","back-end","real-time","efficient","front-end",
	"distributed","seamless","extensible","turn-key","world-class",
	"open-source","cross-platform","cross-media","synergistic",
	"bricks-and-clicks","out-of-the-box","enterprise","integrated",
	"impactful","wireless","transparent","next-generation","cutting-edge",
	"user-centric","visionary","customized","ubiquitous","plug-and-play",
	"collaborative","compelling","holistic","rich"]

noun = ["synergies","web-readiness","paradigms","markets","partnerships",
	"infrastructures","platforms", "initiatives","channels","eyeballs",
	"communities","ROI","solutions","e-tailers","e-services","action-items",
	"portals","niches","technologies","content","vortals","supply-chains",
	"convergence","relationships", "architectures","interfaces","e-markets",
	"e-commerce","systems","bandwidth","infomediaries","models","mindshare",
	"deliverables","users","schemas","networks","applications","metrics",
	"e-business","functionalities","experiences","web services",
	"methodologies"]

def rarr(arr):
   return randint(0,len(arr)-1)

def handle_buzz(bot, ievent):
   ievent.reply("%s %s %s" % (verb[rarr(verb)],adj[rarr(adj)],noun[rarr(noun)]))


cmnds.add('buzz', handle_buzz, 'USER')
examples.add('buzz', 'generate buzz', 'buzz')



# Description: freetranslation.com parser
# Author: Wijnand 'tehmaze' Modderman - http://tehmaze.com
# License: BSD
# License: For non-commercial use only: http://www.freetranslation.com/terms.htm

import httplib
import re
import types
import urllib
import urlparse
from gozerbot.commands import cmnds
from gozerbot.config import config
from gozerbot.examples import examples
from gozerbot.generic import geturl, posturl, waitforqueue
from gozerbot.plughelp import plughelp
# compatibility check (since r1701)
try:
    from gozerbot.textutils import html_unescape
except ImportError:
    import htmllib

    def html_unescape(s):
        p = htmllib.HTMLParser(None)
        p.save_bgn()
        p.feed(s)
        return p.save_end()

plughelp.add('translate', 'translate between languages')

class TranslateLanguageException(Exception):
    pass

class TranslateLoginException(Exception):
    pass

class Translate:
    
    lang = ['English/Spanish', 'English/French', 'English/German', 'English/Italian', 'English/Dutch', 'English/Portuguese', 'English/Russian', 'English/Norwegian', 'English/SimplifiedChinese', 'English/TraditionalChinese', 'English/Japanese', 'Spanish/English', 'French/English', 'German/English', 'Italian/English', 'Dutch/English', 'Portuguese/English', 'Russian/English', 'Japanese/English']
    re_username = re.compile('<input type="hidden" name="username" value="([^"]+)" />', re.I | re.M)
    re_password = re.compile('<input type="hidden" name="password" value="([^"]+)" />', re.I | re.M)
    
    def translate(self, lang_from, lang_to, text):
        # fix case stuff
        lang_from = lang_from.title().replace('chinese', 'Chinese')
        lang_to   = lang_to.title().replace('chinese', 'Chinese')
        if lang_from == 'Chinese':
            lang_from = 'SimplifiedChinese'
        if lang_to == 'Chinese':
            lang_to = 'SimplifiedChinese'
        lang = '%s/%s' % (lang_from, lang_to)
        if not lang in self.lang:
            raise TranslateLanguageException('no such language combination')
        else:
            # oh, this is not needed, ah well, in case we'll ever need it .. ;-)
            #credentials = self.fetch_credentials()
            #if not credentials:
            #    raise TranslateLoginException('Could not fetch username and password')
            #return self.translate_web(credentials, lang, text)
            return self.translate_web({}, lang, text)

    def fetch_credentials(self):
        """On the front page, a username and password are generated and have a limited validity."""
        html = geturl('http://www.freetranslation.com')
        test_username = self.re_username.search(html)
        test_password = self.re_password.search(html)
        credentials = {}
        if test_username and test_password:
            credentials['username'] = test_username.group(1)
            credentials['password'] = test_password.group(1)
        return credentials

    def translate_web(self, postdata, lang, text):
        # defaults
        postdata.update({'language': lang, 'sequence': 'core', 'mode': 'html',
            'template': 'results_en-us.htm', 'charset': 'UTF-8', 'a': 'a',
            'e': 'e', 'i': 'i', 'o': 'o', 'u': 'u', 'misc': 'misc', 
            'Submit': 'FREE Translation'})
        # text
        postdata.update({'srctext': text})
        # pick correct server
        server = 'ets.freetranslation.com'
        if 'chinese' in lang.lower() or 'russian' in lang.lower():
            server = 'ets6.freetranslation.com'
        elif 'japanese' in lang.lower():
            server = 'tets9.freetranslation.com'
        result = posturl('http://%s/' % server, {}, postdata)
        for line in result.read().splitlines():
            # <textarea name="dsttext" cols="40" rows="6">ff test or works this</textarea><br />
            if 'name="dsttext"' in line:
                return line.split('>')[1].split('</textarea')[0]
        return None

translate = Translate()

def handle_translate(bot, ievent):
    if ievent.inqueue:
        text = ' '.join(waitforqueue(ievent.inqueue, 5))
    elif len(ievent.args) < 3:
        ievent.missing('<from language> <to language> <text>')
        return
    else:
        text = ' '.join(ievent.args[2:])
    try:
        result = translate.translate(ievent.args[0], ievent.args[1], text)
        if result:
            ievent.reply(html_unescape(result))
        else:
            ievent.reply('translation failed (no result)')
    except TranslateLanguageException, e:
        ievent.reply(str(e))

cmnds.add('translate', handle_translate, 'USER')
examples.add('translate', 'translate between languages', 'translate English Dutch how do you do')

def handle_translate_langs(bot, ievent):
    langs = []
    for lang in translate.lang:
        lang = lang.split('/')
        for l in lang:
            if not l in langs:
                langs.append(l)
    langs.sort()
    ievent.reply('available languages: %s' % ', '.join(langs))

cmnds.add('translate-lang', handle_translate_langs, 'USER')

trmap = {
    'de': 'German',
    'en': 'English',
    'es': 'Spanish',
    'fr': 'French',
    'it': 'Italian',
    'ja': 'Japanese',
    'jp': 'Japanese',
    'nl': 'Dutch',
    'no': 'Norwegian',
    'pt': 'Portuguese',
    'ru': 'Russian',
    'cn': 'SimplifiedChinese'
    }
re_tr = re.compile('^(\S\S)[>\s](\S\S)')

def handle_tr(bot, ievent):
    if not re_tr.match(ievent.txt.strip()):
        return ievent.missing('<from lang>to lang> <text>')
    else:
        if ievent.inqueue:
            text = ' '.join(waitforqueue(ievent.inqueue, 5))
        elif '>' in ievent.args[0]:
            text = ' '.join(ievent.args[1:])
        else:
            text = ' '.join(ievent.args[2:])
        test_tr = re_tr.search(ievent.args[0])
        if not test_tr or not trmap.has_key(test_tr.group(1).lower()) or \
             not trmap.has_key(test_tr.group(2).lower()):
            langs = trmap.keys()
            langs.sort()
            return ievent.reply('invalid language combination, available languages are: %s' % ', '.join(langs))
        lang1 = trmap[test_tr.group(1).lower()]
        lang2 = trmap[test_tr.group(2).lower()]
        try:
            result = translate.translate(lang1, lang2, text)
            if result:
                ievent.reply(html_unescape(result))
            else:
                ievent.reply('translation failed (no result)')
        except TranslateLanguageException, e:
            ievent.reply(str(e))

cmnds.add('tr', handle_tr, 'USER')
examples.add('tr', 'translate between two languages', 'tr nl>en het is mooi weer vandaag')



# plugs/karma.py
#
#

""" karma plugin """

__copyright__ = 'this file is in the public domain'
__depending__ = ['fans', 'quote']

from gozerbot.tests import tests
from gozerbot.commands import cmnds
from gozerbot.examples import examples
from gozerbot.redispatcher import rebefore
from gozerbot.datadir import datadir
from gozerbot.generic import handle_exception, rlog, lockdec
from gozerbot.utils.statdict import Statdict
from gozerbot.aliases import aliases
from gozerbot.plughelp import plughelp
from gozerbot.config import config
import thread, pickle, time, os

from gozerbot.database.alchemy import Base, create_session, trans, create_all, engine
from datetime import datetime
from time import localtime
from sqlalchemy import Column, String, Integer, Text, DateTime, ForeignKey, Sequence
import sqlalchemy as sa

class Karma(Base):
    __tablename__ = 'karma'
    __table_args__ = {'useexisting': True}
    item = Column('item', String(255), primary_key=True)
    value = Column('value', Integer, nullable=False)

    def __init__(self, item, value):
        self.item = item
        self.value = value

class WhyKarma(Base):
    __tablename__ = 'whykarma'
    __table_args__ = {'useexisting': True}
    item = Column('item', String(255), nullable=False)
    updown = Column('updown', String(10), nullable=False)
    why = Column('why', Text, nullable=False)
    __mapper_args__ = {'primary_key':[item,updown,why]}

    def __init__(self, item, updown, why):
        self.item = item
        self.updown = updown
        self.why = why

class WhoKarma(Base):
    __tablename__ = 'whokarma'
    __table_args__ = {'useexisting': True}
    item = Column('item', String(255), nullable=False)
    nick = Column('nick', String(255), nullable=False)
    updown = Column('updown', String(10), nullable=False)
    __mapper_args__ = {'primary_key': [item, nick, updown]}

    def __init__(self, item, nick, updown):
        self.item = item
        self.nick = nick
        self.updown = updown

## UPGRADE PART

def upgrade():
    teller = 0
    oldfile = datadir + os.sep + 'old'
    dbdir = datadir + os.sep + 'db' + os.sep + 'karma.db'
    if os.path.exists(dbdir):
        try:
            from gozerbot.database.db import Db
            db = Db(dbtype='sqlite')
            db.connect('db/karma.db')
            result = db.execute(""" SELECT * FROM karma """)
            if result:
                for i in result:
                    karma.add(i[0], i[1])
                    teller += 1
            result = db.execute(""" SELECT * FROM whokarma """)
            if result:
                for i in result:
                    if i[2] == 'up':
                        karma.setwhoup(i[0], i[1])
                    else:
                        karma.setwhodown(i[0], i[1])
            result = db.execute(""" SELECT * FROM whykarma """)
            if result:
                for i in result:
                    karma.addwhy(*i)
        except Exception, ex:
            handle_exception()
        return teller

    try:
        from gozerbot.utils.generic import dosed
        from gozerbot.database.db import Db
        from gozerbot.compat.karma import Karma

        oldkarma = Karma(oldfile) 

        if not oldkarma:
            return

        k = oldkarma
        got = []

        rlog(10, 'karma', 'upgrading karma items')
        for i,j in k.karma.iteritems():
            if i in got:
                continue
            else:
                got.append(i)
                try:
                    karma.add(i,j)
                    teller += 1
                except sa.exc.IntegrityError, ex:
                    #if 'not unique' in str(ex):
                    #    continue
                    raise

        rlog(10, 'karma', 'upgrading reasonup')
        for i,j in k.reasonup.iteritems():
            for z in j:
                karma.addwhy(i, 'up', z)

        rlog(10, 'karma', 'upgrading reasondown')
        for i,j in k.reasondown.iteritems():
            for z in j:
                karma.addwhy(i, 'down', z)

        rlog(10, 'karma', 'upgrading whodown')
        for i,j in k.whodown.iteritems():
            for z in j:
                karma.setwhodown(i, z)

        rlog(10, 'karma', 'upgrading whoup')
        for i,j in k.whoup.iteritems():
            for z in j:
                karma.setwhoup(i, z)

    except Exception, ex:
        handle_exception()
        rlog(10, 'karma', 'failed to upgrade: %s' % str(ex))
        return
    rlog(10, 'karma', 'upgraded %s karma items' % str(teller))

# END UPGRADE PART

plughelp.add('karma', 'maintain karma of items .. use ++ to raise karma by 1 \
or use -- to lower by 1 .. reason might be given after a "#"')

class KarmaDb(object):

    """ karma object """

    def save(self):
        pass

    def size(self):
        """ return number of items """
        s = create_session()
        count = s.query(sa.func.count(Karma.item)).first()[0]
        return count

    @trans
    def add(self, session, item, value):
        item = item.lower()
        if session.query(Karma).filter(Karma.item==item).count():
            rlog(10, 'karma', '%s item is already in database' % item)
            return
        k = Karma(item, value)
        session.add(k)
        return 1

    def get(self, item):
        """ get karma of item """
        item = item.lower()
        s = create_session()
        karma = s.query(Karma).filter(Karma.item==item).first()
        if karma:
            return karma.value

    @trans
    def delete(self, session, item):
        """ delete karma item """
        item = item.lower()
        karma = session.query(Karma).filter(Karma.item==item).first()
        if karma:
            session.delete(karma)
            return 1

    @trans
    def addwhy(self, session, item, updown, reason):
        """ add why of karma up/down """
        item = item.lower()
        whykarma = WhyKarma(item, updown, reason)
        session.add(whykarma)

    @trans
    def upitem(self, session, item, reason=None):
        """ up a karma item with/without reason """
        item = item.lower()
        karma = session.query(Karma).filter(Karma.item==item).first()
        if not karma:
            karma = Karma(item, 0)
            session.add(karma)
        karma.value = karma.value + 1
        if reason:
            whykarma = WhyKarma(item, 'up', reason.strip())
            session.add(whykarma)

    @trans
    def down(self, session, item, reason=None):
        """ lower a karma item with/without reason """
        item = item.lower()
        s = create_session()
        karma = s.query(Karma).filter(Karma.item==item).first()
        if not karma:
            karma = Karma(item, 0)
            session.add(karma)
        karma.value = karma.value - 1
        if reason:
            whykarma = WhyKarma(item, 'down', reason.strip())
            session.add(whykarma)

    def whykarmaup(self, item):
        """ get why of karma ups """
        s = create_session()
        item = item.lower()
        karma = s.query(WhyKarma).filter(WhyKarma.item==item).filter(WhyKarma.updown=='up')
        if karma:
            return [k.why for k in karma]

    def whykarmadown(self, item):
        """ get why of karma downs """
        s = create_session()
        item = item.lower()
        karma = s.query(WhyKarma).filter(WhyKarma.item==item).filter(WhyKarma.updown=='down')
        if karma:
            return [k.why for k in karma]

    @trans
    def setwhoup(self, session, item, nick):
        """ set who upped a karma item """
        item = item.lower()
        nick = nick.lower()
        wk = WhoKarma(item, nick, 'up')
        session.add(wk)
        session.commit()

    @trans
    def setwhodown(self, session, item, nick):
        """ set who lowered a karma item """
        item = item.lower()
        nick = nick.lower()
        wk = WhoKarma(item, nick, 'down')
        session.add(wk)
        session.commit()

    def getwhoup(self, item):
        """ get list of who upped a karma item """
        item = item.lower()
        s = create_session()
        karma = s.query(WhoKarma).filter(WhoKarma.item==item).filter(WhoKarma.updown=='up').all()
        if karma:
            return [k.nick for k in karma]

    def getwhodown(self, item):
        """ get list of who downed a karma item """
        item = item.lower()
        s = create_session()
        karma = s.query(WhoKarma).filter(WhoKarma.item==item).filter(WhoKarma.updown=='down').all()
        if karma:
            return [k.nick for k in karma]

    def search(self, item):
        """ search for matching karma item """
        item = item.lower()
        s = create_session()
        karma = s.query(Karma).filter(Karma.item.like('%%%s%%' % item))
        if karma:
            return karma

    def good(self, limit=10):
        """ show top 10 of karma items """
        statdict = Statdict()
        s = create_session()
        karma = s.query(Karma).all()
        if not karma:
            return []
        for i in karma:
            if i.item.startswith('quote '):
                continue
            statdict.upitem(i.item, i.value)
        return statdict.top(limit=limit)

    def bad(self, limit=10):
        """ show lowest 10 of negative karma items """
        statdict = Statdict()
        s = create_session()
        karma = s.query(Karma).all()
        if not karma:
            return []
        for i in karma:
            if i.item.startswith('quote '):
                continue
            statdict.upitem(i.item, i.value)
        return statdict.down(limit=limit)

    def quotegood(self, limit=10):
        """ show top 10 of karma items """
        statdict = Statdict()
        s = create_session()
        karma = s.query(Karma).all()
        if not karma:
            return []
        for i in karma:
            if not i.item.startswith('quote '):
                continue
            statdict.upitem(i.item, i.value)
        return statdict.top(limit=limit)

    def quotebad(self, limit=10):
        """ show lowest 10 of negative karma items """
        statdict = Statdict()
        s = create_session()
        karma = s.query(Karma).all()
        if not karma:
            return []
        for i in karma:
            if not i.item.startswith('quote '):
                continue
            statdict.upitem(i.item, i.value)
        return statdict.down(limit=limit)

    def whatup(self, nick):
        """ show what items are upped by nick """
        nick = nick.lower()
        statdict = Statdict()
        s = create_session()
        whokarma = s.query(WhoKarma).filter(WhoKarma.nick==nick).filter(WhoKarma.updown=='up')
        if not whokarma:
            return []
        for i in whokarma:
            statdict.upitem(i.item)
        return statdict.top()

    def whatdown(self, nick):
        """ show what items are upped by nick """
        nick = nick.lower()
        statdict = Statdict()
        s = create_session()
        whokarma = s.query(WhoKarma).filter(WhoKarma.nick==nick).filter(WhoKarma.updown=='down')
        if not whokarma:
            return []
        for i in whokarma:
            statdict.upitem(i.item)
        return statdict.top()

karma = KarmaDb()
create_all('karma')
assert(karma)

ratelimited = []
limiterlock = thread.allocate_lock()
limlock = lockdec(limiterlock)
nolimiter = False

def size():
    """ return number of kamra items """
    return karma.size()

def search(what, queue):
    rlog(10, 'karma', 'searched for %s' % what)
    result = karma.search(what)
    for i in result:
        queue.put_nowait("%s has karma %s" % (i.item, i.value))

@limlock
def ratelimit(bot, ievent):
    """ karma rate limiter """
    if nolimiter:
        return 1
    waittime = 30
    limit = 2
    try:
        name = ievent.userhost
        # Create a state for this user and his/her karma-state if necessary
        if not bot.state.has_key(name):
            bot.state[name] = {}
        if not bot.state[name].has_key('karma'):
            bot.state[name]['karma'] = {'count': 0, 'time': time.time() } 
        # If the waittime has elapsed, reset the counter
        if time.time() > (bot.state[name]['karma']['time'] + waittime):
            bot.state[name]['karma']['count'] = 0 
        # Update counter
        bot.state[name]['karma']['count'] += 1
        # If counter is too high, limit :)
        if bot.state[name]['karma']['count'] > limit:
            if name in ratelimited:
                return 0
            ievent.reply("karma limit reached, you'll have to wait %s \
seconds" % int((bot.state[name]['karma']['time'] + waittime) - time.time()))
            ratelimited.append(name)
            return 0
        # Update time
        bot.state[name]['karma']['time'] = time.time()
        # Ratelimiting passed :)
        try:
            ratelimited.remove(name)
        except ValueError:
            pass
        return 1
    except Exception, ex:
        handle_exception(ievent)

def handle_karmaget(bot, ievent):
    """ karma-get <item> .. show karma of item """
    if not ievent.rest:
        ievent.missing('<item>')
        return
    else:
        item = ievent.rest
    result = karma.get(item)
    if result:
        ievent.reply("%s has karma %s" % (item, str(result)))
    else:
        ievent.reply("%s has no karma yet" % item)

cmnds.add('karma-get', handle_karmaget, ['USER', 'WEB', 'ANONKARMA', 'CLOUD'])
examples.add('karma-get', 'karma-get <item> .. show karma of <item>', \
'karma-get dunker')
aliases.data['karma'] = 'karma-get'

def nolimit():
    global nolimiter
    nolimiter = True

def dolimit():
    global nolimiter
    nolimiter = False

tests.start(nolimit).add('gozerbot++').add('karma-get gozerbot', 'gozerbot has karma (\d+)').end(dolimit)

def handle_karmadel(bot, ievent):
    """ karma-del <item> .. delete karma item """
    if not ievent.rest:
        ievent.missing('<item>')
        return
    item = ievent.rest
    result = karma.delete(item)
    if result:
        ievent.reply("%s deleted" % item)
    else:
        ievent.reply("can't delete %s" % item)

cmnds.add('karma-del', handle_karmadel, ['OPER'])
examples.add('karma-del', 'karma-del <item> .. delete karma item', \
'karma-del dunker')
tests.add('boozer--').add('karma-del boozer', 'boozer deleted')

def handle_karmaup(bot, ievent):
    """ <item>++ ['#' <reason>] .. increase karma of item with optional \
        reason """
    if not ratelimit(bot, ievent):
        return
    (item, reason) = ievent.groups
    item = item.strip().lower()
    if not item:
        return
    karma.upitem(item, reason=reason)
    karma.setwhoup(item, ievent.nick)
    ievent.reply('karma of '+ item + ' is now ' + str(karma.get(item)))

rebefore.add(8, '^(.+)\+\+\s+#(.*)$', handle_karmaup, ['USER', 'KARMA', \
'ANONKARMA'],  allowqueue=False)
examples.add('++', "<item>++ ['#' <reason>] .. higher karma of item with 1 \
(use optional reason)", '1) gozerbot++ 2) gozerbot++ # top bot')
tests.add('gozerbot++', 'karma of gozerbot is now (\d+)')
tests.add('gozerbot++ # top bot', 'karma of gozerbot is now (\d+)')

def handle_karmaup2(bot, ievent):
    """ increase karma without reason """
    ievent.groups += [None, ]
    handle_karmaup(bot, ievent)

rebefore.add(9, '^(.+)\+\+$', handle_karmaup2, ['USER', 'ANONKARMA'], \
allowqueue=False)

def handle_karmadown(bot, ievent):
    """ <item>-- ['#' <reason> .. decrease karma item with reason """
    if not ratelimit(bot, ievent):
        return
    (item, reason) = ievent.groups
    item = item.strip().lower()
    if not item:
        return
    karma.down(item, reason=reason)
    karma.setwhodown(item, ievent.nick)
    ievent.reply('karma of ' + item + ' is now ' + str(karma.get(item)))

rebefore.add(8, '^(.+)\-\-\s+#(.*)$', handle_karmadown, ['USER', 'KARMA', \
'ANONKARMA'], allowqueue=False)
examples.add('--', "<item>-- ['#' <reason>] .. lower karma of item with 1 \
(use optional reason)", '1) gozerbot-- 2) gozerbot-- # bad bot')
tests.add('miep--', 'karma of miep is now (-\d+)')
tests.add('miep-- # top bot', 'karma of miep is now (-\d+)')

def handle_karmadown2(bot, ievent):
    """ decrease karma item without reason """
    ievent.groups += [None, ]
    handle_karmadown(bot, ievent)

rebefore.add(9, '^(.+)\-\-$', handle_karmadown2, ['USER', 'KARMA', \
'ANONKARMA'], allowqueue=False)

def handle_karmawhyup(bot, ievent):
    """ karma-whyup <item> .. show why karma of item has been increased """
    if not ievent.rest:
        ievent.missing('<item>')
        return
    item = ievent.rest
    result = karma.whykarmaup(item)
    if result:
        ievent.reply('whykarmaup of %s: ' % item, result, dot=True)
    else:
        ievent.reply('%s has no reason for karmaup yet' % item)

cmnds.add('karma-whyup', handle_karmawhyup, ['USER', 'WEB', \
'ANONKARMA'])
examples.add('karma-whyup', 'karma-whyup <item> .. show the reason why \
karma of <item> was raised', 'karma-whyup gozerbot')
aliases.data['wku'] = 'karma-whyup'
tests.add('gozerbot++ # top bot').add('karma-whyup gozerbot', 'top bot')

def handle_whykarmadown(bot, ievent):
    """ karma-whydown <item> .. show why karma of item has been decreased """
    if not ievent.rest:
        ievent.missing('<item>')
        return
    item = ievent.rest
    result = karma.whykarmadown(item)
    if result:
        ievent.reply('whykarmadown of %s: ' % item, result, dot=True)
    else:
        ievent.reply('%s has no reason for karmadown yet' % item)

cmnds.add('karma-whydown', handle_whykarmadown, ['USER', 'WEB', \
'ANONKARMA'])
examples.add('karma-whydown', 'karma-whydown <item> .. show the reason why \
karma of <item> was lowered', 'karma-whydown gozerbot')
aliases.data['wkd'] = 'karma-whydown'
tests.add('miep-- # kut bot').add('karma-whydown miep', 'kut bot')

def handle_karmagood(bot, ievent):
    """ karma-good .. show top 10 karma items """
    result = karma.good(limit=10)
    if result:
        res = []
        for i in result:
            if i[1] > 0:
                res.append("%s=%s" % (i[0], i[1]))
        ievent.reply('goodness: ', res, dot=True)
    else:
        ievent.reply('karma void')

cmnds.add('karma-good', handle_karmagood, ['USER', 'WEB', 'ANONKARMA', 'CLOUD'])
examples.add('karma-good', 'show top 10 karma', 'karma-good')
aliases.data['good'] = 'karma-good'
tests.add('gozerbot++').add('karma-good', 'gozerbot')

def handle_karmabad(bot, ievent):
    """ karma-bad .. show 10 most negative karma items """
    result = karma.bad(limit=10)
    if result:
        res = []
        for i in result:
            if i[1] < 0:
                res.append("%s=%s" % (i[0], i[1]))
        ievent.reply('badness: ', res, dot=True)
    else:
        ievent.reply('karma void')

cmnds.add('karma-bad', handle_karmabad, ['USER', 'WEB', 'ANONKARMA', 'CLOUD'])
examples.add('karma-bad', 'show lowest top 10 karma', 'karma-bad')
aliases.data['bad'] = 'karma-bad'
tests.add('miep--').add('karma-bad', 'miep')

def handle_whokarmaup(bot, ievent):
    """ karma-whoup <item> .. show who increased a karma item """
    if not ievent.rest:
        ievent.missing('<item>')
        return
    item = ievent.rest
    result = karma.getwhoup(item)
    statdict = Statdict()
    if result:
        for i in result:
            statdict.upitem(i)
        res = []
        for i in statdict.top():
            res.append("%s=%s" % i)
        ievent.reply("whokarmaup of %s: " % item, res, dot=True)
    else:
        ievent.reply('no whokarmaup data available for %s' % item)

cmnds.add('karma-whoup', handle_whokarmaup, ['USER', 'WEB', \
'ANONKARMA', 'CLOUD'])
examples.add('karma-whoup', 'karma-whoup <item> .. show who raised the \
karma of <item>', 'karma-whoup gozerbot')
tests.add('gozerbot++').add('karma-whoup gozerbot', '{{ me }}')

def handle_whokarmadown(bot, ievent):
    """ karma-whodown <item> .. show who decreased a karma item """
    if not ievent.rest:
        ievent.missing('<item>')
        return
    item = ievent.rest
    result = karma.getwhodown(item)
    statdict = Statdict()
    if result:
        for i in result:
            statdict.upitem(i)
        res = []
        for i in statdict.top():
            res.append("%s=%s" % i)
        ievent.reply("whokarmadown of %s: " % item, res, dot=True)
    else:
        ievent.reply('no whokarmadown data available for %s' % item)

cmnds.add('karma-whodown', handle_whokarmadown, ['USER', 'WEB', \
'ANONKARMA', 'CLOUD'])
examples.add('karma-whodown', 'karma-whodown <item> .. show who lowered \
the karma of <item>', 'karma-whodown gozerbot')
tests.add('miep--').add('karma-whodown miep', '{{ me }}')

def handle_karmasearch(bot, ievent):
    """ karma-search <txt> .. search for karma items """
    what = ievent.rest
    if not what:
        ievent.missing('<txt>')
        return
    result = karma.search(what)
    if result:
        res = []
        for i in result:
            res.append("%s (%s)" % (i.item, i.value))
        ievent.reply("karma items matching %s: " % what, res, dot=True)
    else:
        ievent.reply('no karma items matching %s found' % what)

cmnds.add('karma-search', handle_karmasearch, ['USER', 'WEB', \
'ANONKARMA', 'CLOUD'])
examples.add('karma-search', 'karma-search <txt> .. search karma' , \
'karma-search gozerbot')
tests.add('gozerbot++').add('karma-search gozer', 'gozerbot')

def handle_karmawhatup(bot, ievent):
    """ show what karma items have been upped by nick """
    try:
        nick = ievent.args[0]
    except IndexError:
        ievent.missing('<nick>')
        return
    result = karma.whatup(nick)
    if result:
        res = []
        for i in result:
            res.append("%s (%s)" % i)
        ievent.reply("karma items upped by %s: " % nick, res, dot=True)
    else:
        ievent.reply('no karma items upped by %s' % nick)

cmnds.add('karma-whatup', handle_karmawhatup, ['USER', 'WEB', \
'ANONKARMA', 'CLOUD'])
examples.add('karma-whatup', 'karma-whatup <nick> .. show what karma items \
<nick> has upped' , 'karma-whatup dunker')
tests.add('gozerbot++').add('karma-whatup {{ me }}', 'gozerbot')

def handle_karmawhatdown(bot, ievent):
    """ show what karma items have been lowered by nick """
    try:
        nick = ievent.args[0]
    except IndexError:
        ievent.missing('<nick>')
        return
    result = karma.whatdown(nick)
    if result:
        res = []
        for i in result:
            res.append("%s (%s)" % i)
        ievent.reply("karma items downed by %s: " % nick, res, dot=True)
    else:
        ievent.reply('no karma items downed by %s' % nick)

cmnds.add('karma-whatdown', handle_karmawhatdown, ['USER', 'WEB', \
'ANONKARMA', 'CLOUD'])
examples.add('karma-whatdown', 'karma-whatdown <nick> .. show what karma \
items <nick> has downed' , 'karma-whatdown dunker')
tests.add('miep--').add('karma-whatdown {{ me }}', 'miep')



# plugs/wisdom.py
#
#

__copyright__ = 'this file is in the public domain'
__thnx__ = 'thnx to snore for this one'

from gozerbot.commands import cmnds
from gozerbot.examples import examples
from gozerbot.plughelp import plughelp
import os, string, random

plughelp.add('wisdom', 'show some wisdom')

excuses = ['booo', ]
matrix = ['booo', ]
motivation = ['booo', ]

def init():
    global excuses
    global matrix
    global motivation
    excuses = excusestxt.splitlines()
    matrix = matrixtxt.splitlines()
    motivation = motivationtxt.splitlines()
    return 1

def handle_matrix(bot, ievent):
    """ get a matrix quote """
    rand = random.randint(1,len(matrix))
    ievent.reply(matrix[rand-1].strip())

def handle_excuse(bot, ievent):
    """ get an excuse """
    rand = random.randint(1,len(excuses))
    ievent.reply(excuses[rand-1].strip())

def handle_motivation(bot, ievent):
    """ get despair  """
    rand = random.randint(1,len(motivation))
    ievent.reply(motivation[rand-1].strip())


cmnds.add('matrix', handle_matrix, 'USER')
examples.add('matrix', 'get a matrix quote', 'matrix')
cmnds.add('excuse', handle_excuse, 'USER')
examples.add('excuse', 'get an excuse', 'excuse')
cmnds.add('motivation', handle_motivation, 'USER')
examples.add('motivation', 'get motivated', 'motivation')

excusestxt = """
clock speed
solar flares
electromagnetic radiation from satellite debris
static from nylon underwear
static from plastic slide rules
global warming
poor power conditioning
static buildup
doppler effect
hardware stress fractures
magnetic interferance from money/credit cards
dry joints on cable plug
we're waiting for [the phone company] to fix that line
sounds like a Windows problem, try calling Microsoft support
temporary routing anomoly
somebody was calculating pi on the server
fat electrons in the lines
excess surge protection
floating point processor overflow
divide-by-zero error
POSIX complience problem
monitor resolution too high
improperly oriented keyboard
network packets travelling uphill (use a carrier pigeon)
Decreasing electron flux
first Saturday after first full moon in Winter
radiosity depletion
CPU radiator broken
It works the way the Wang did, what's the problem
positron router malfunction
cellular telephone interference
techtonic stress
pizeo-electric interference
(l)user error
working as designed
dynamic software linking table corrupted
heavy gravity fluctuation, move computer to floor rapidly
secretary plugged hairdryer into UPS
terrorist activities
not enough memory, go get system upgrade
interrupt configuration error
spaghetti cable cause packet failure
boss forgot system password
bank holiday - system operating credits  not recharged
virus attack, luser responsible
waste water tank overflowed onto computer
Complete Transient Lockout
bad ether in the cables
Bogon emissions
Change in Earth's rotational speed
Cosmic ray particles crashed through the hard disk platter
Smell from unhygenic janitorial staff wrecked the tape heads
Little hamster in running wheel had coronary; waiting for replacement to be Fedexed from Wyoming
Evil dogs hypnotized the night shift
Plumber mistook routing panel for decorative wall fixture
Electricians made popcorn in the power supply
Groundskeepers stole the root password
high pressure system failure
failed trials, system needs redesigned
system has been recalled
not approved by the FCC
need to wrap system in aluminum foil to fix problem
not properly grounded, please bury computer
CPU needs recalibration
system needs to be rebooted
bit bucket overflow
descramble code needed from software company
only available on a need to know basis
knot in cables caused data stream to become twisted and kinked
nesting roaches shorted out the ether cable
The file system is full of it
Satan did it
Daemons did it
You're out of memory
There isn't any problem
Unoptimized hard drive
Typo in the code
Yes, yes, its called a desgin limitation
Look, buddy:  Windows 3.1 IS A General Protection Fault.
That's a great computer you have there; have you considered how it would work as a BSD machine?
Please excuse me, I have to circuit an AC line through my head to get this database working.
Yeah, yo mama dresses you funny and you need a mouse to delete files.
Support staff hung over, send aspirin and come back LATER.
Someone is standing on the ethernet cable, causeing a kink in the cable
Windows 95 undocumented "feature"
Runt packets
Password is too complex to decrypt
Boss' kid fucked up the machine
Electromagnetic energy loss
Budget cuts
Mouse chewed through power cable
Stale file handle (next time use Tupperware(tm)!)
Feature not yet implimented
Internet outage
Pentium FDIV bug
Vendor no longer supports the product
Small animal kamikaze attack on power supplies
The vendor put the bug there.
SIMM crosstalk.
IRQ dropout
Collapsed Backbone
Power company testing new voltage spike (creation) equipment
operators on strike due to broken coffee machine
backup tape overwritten with copy of system manager's favourite CD
UPS interrupted the server's power
The electrician didn't know what the yellow cable was so he yanked the ethernet out.
The keyboard isn't plugged in
The air conditioning water supply pipe ruptured over the machine room
The electricity substation in the car park blew up.
The rolling stones concert down the road caused a brown out
The salesman drove over the CPU board.
The monitor is plugged into the serial port
Root nameservers are out of sync
electro-magnetic pulses from French above ground nuke testing.
your keyboard's space bar is generating spurious keycodes.
the real ttys became pseudo ttys and vice-versa.
the printer thinks its a router.
the router thinks its a printer.
evil hackers from Serbia.
we just switched to FDDI.
halon system went off and killed the operators.
because Bill Gates is a Jehovah's witness and so nothing can work on St. Swithin's day.
user to computer ratio too high.
user to computer ration too low.
we just switched to Sprint.
it has Intel Inside
Sticky bits on disk.
Power Company having EMP problems with their reactor
The ring needs another token
new management
telnet: Unable to connect to remote host: Connection refused
SCSI Chain overterminated
It's not plugged in.
because of network lag due to too many people playing deathmatch
You put the disk in upside down.
Daemons loose in system.
User was distributing pornography on server; system seized by FBI.
BNC (brain not  (user brain not connected)
UBNC (user brain not connected)
LBNC (luser brain not connected)
disks spinning backwards - toggle the hemisphere jumper.
new guy cross-connected phone lines with ac power bus.
had to use hammer to free stuck disk drive heads.
Too few computrons available.
Flat tire on station wagon with tapes.  ("Never underestimate the bandwidth of a station wagon full of tapes hurling down the highway" Andrew S. Tanenbaum) 
Communications satellite used by the military for star wars.
Party-bug in the Aloha protocol.
Insert coin for new game
Dew on the telephone lines.
Arcserve crashed the server again.
Some one needed the powerstrip, so they pulled the switch plug.
My pony-tail hit the on/off switch on the power strip.
Big to little endian conversion error
You can tune a file system, but you can't tune a fish (from most tunefs man pages)
Dumb terminal
Zombie processes haunting the computer
Incorrect time syncronization
Defunct processes
Stubborn processes
non-redundant fan failure 
monitor VLF leakage
bugs in the RAID
no "any" key on keyboard
root rot
Backbone Scoliosis
/pub/lunch
excessive collisions & not enough packet ambulances
le0: no carrier: transceiver cable problem?
broadcast packets on wrong frequency
popper unable to process jumbo kernel
NOTICE: alloc: /dev/null: filesystem full
pseudo-user on a pseudo-terminal
Recursive traversal of loopback mount points
Backbone adjustment
OS swapped to disk
vapors from evaporating sticky-note adhesives
sticktion
short leg on process table
multicasts on broken packets
ether leak
Atilla the Hub
endothermal recalibration
filesystem not big enough for Jumbo Kernel Patch
loop found in loop in redundant loopback
system consumed all the paper for paging
permission denied
Reformatting Page. Wait...
..disk or the processor is on fire.
SCSI's too wide.
Proprietary Information.
Just type 'mv * /dev/null'.
runaway cat on system.
Did you pay the new Support Fee?
We only support a 1200 bps connection.
We only support a 28000 bps connection.
Me no internet, only janitor, me just wax floors.
I'm sorry a pentium won't do, you need an SGI to connect with us.
Post-it Note Sludge leaked into the monitor.
the curls in your keyboard cord are losing electricity.
The monitor needs another box of pixels.
RPC_PMAP_FAILURE
kernel panic: write-only-memory (/dev/wom0) capacity exceeded.
Write-only-memory subsystem too slow for this machine. Contact your local dealer.
Just pick up the phone and give modem connect sounds. "Well you said we should get more lines so we don't have voice lines."
Quantum dynamics are affecting the transistors
Police are examining all internet packets in the search for a narco-net-traficer
We are currently trying a new concept of using a live mouse.  Unfortuantely, one has yet to survive being hooked up to the computer.....please bear with us.
Your mail is being routed through Germany ... and they're censoring us.
Only people with names beginning with 'A' are getting mail this week (a la Microsoft)
We didn't pay the Internet bill and it's been cut off.
Lightning strikes.
Of course it doesn't work. We've performed a software upgrade.
Change your language to Finnish.
Flourescent lights are generating negative ions. If turning them off doesn't work, take them out and put tin foil on the ends.
High nuclear activity in your area.
What office are you in? Oh, that one.  Did you know that your building was built over the universities first nuclear research site? And wow, are'nt you the lucky one, your office is right over where the core is buried!
The MGs ran out of gas.
The UPS doesn't have a battery backup.
Recursivity.  Call back if it happens again.
Someone thought The Big Red Button was a light switch.
The mainframe needs to rest.  It's getting old, you know.
I'm not sure.  Try calling the Internet's head office -- it's in the book.
The lines are all busy (busied out, that is -- why let them in to begin with?).
Jan  9 16:41:27 huber su: 'su root' succeeded for .... on /dev/pts/1
It's those computer people in X {city of world}.  They keep stuffing things up.
A star wars satellite accidently blew up the WAN.
Fatal error right in front of screen
That function is not currently supported, but Bill Gates assures us it will be featured in the next upgrade.
wrong polarity of neutron flow
Lusers learning curve appears to be fractal
We had to turn off that service to comply with the CDA Bill.
Ionisation from the air-conditioning
TCP/IP UDP alarm threshold is set too low.
Someone is broadcasting pigmy packets and the router dosn't know how to deal with them.
The new frame relay network hasn't bedded down the software loop transmitter yet. 
Fanout dropping voltage too much, try cutting some of those little traces
Plate voltage too low on demodulator tube
You did wha... oh _dear_....
CPU needs bearings repacked
Too many little pins on CPU confusing it, bend back and forth until 10-20% are neatly removed. Do _not_ leave metal bits visible!
_Rosin_ core solder? But...
Software uses US measurements, but the OS is in metric...
The computer fletely, mouse and all.
Your cat tried to eat the mouse.
The Borg tried to assimilate your system. Resistance is futile.
It must have been the lightning storm we had (yesterdy) (last week) (last month)
Due to Federal Budget problems we have been forced to cut back on the number of users able to access the system at one time. (namely none allowed....)
Too much radiation coming from the soil.
Unfortunately we have run out of bits/bytes/whatever. Don't worry, the next supply will be coming next week.
Program load too heavy for processor to lift.
Processes running slowly due to weak power supply
Our ISP is having {switching,routing,SMDS,frame relay} problems
We've run out of licenses
Interference from lunar radiation
Standing room only on the bus.
You need to install an RTFM interface.
That would be because the software doesn't work.
That's easy to fix, but I can't be bothered.
Someone's tie is caught in the printer, and if anything else gets printed, he'll be in it too.
We're upgrading /dev/null
The Usenet news is out of date
Our POP server was kidnapped by a weasel.
It's stuck in the Web.
Your modem doesn't speak English.
The mouse escaped.
All of the packets are empty.
The UPS is on strike.
Neutrino overload on the nameserver
Melting hard drives
Someone has messed up the kernel pointers
The kernel license has expired
Netscape has crashed
The cord jumped over and hit the power switch.
It was OK before you touched it.
Bit rot
U.S. Postal Service
Your Flux Capacitor has gone bad.
The Dilithium Cyrstals need to be rotated.
The static electricity routing is acting up...
Traceroute says that there is a routing problem in the backbone.  It's not our problem.
The co-locator cannot verify the frame-relay gateway to the ISDN server.
High altitude condensation from U.S.A.F prototype aircraft has contaminated the primary subnet mask. Turn off your computer for 9 days to avoid damaging it.
Lawn mower blade in your fan need sharpening
Electrons on a bender
Telecommunications is upgrading. 
Telecommunications is downgrading.
Telecommunications is downshifting.
Hard drive sleeping. Let it wake up on it's own...
Interference between the keyboard and the chair.
The CPU has shifted, and become decentralized.
Due to the CDA, we no longer have a root account.
We ran out of dial tone and we're and waiting for the phone company to deliver another bottle.
You must've hit the wrong anykey.
PCMCIA slave driver
The Token fell out of the ring. Call us when you find it.
The hardware bus needs a new token.
Too many interrupts
Not enough interrupts
The data on your hard drive is out of balance.
Digital Manipulator exceeding velocity parameters
appears to be a Slow/Narrow SCSI-0 Interface problem
microelectronic Riemannian curved-space fault in write-only file system
fractal radiation jamming the backbone
routing problems on the neural net
IRQ-problems with the Un-Interruptable-Power-Supply
CPU-angle has to be adjusted because of vibrations coming from the nearby road
emissions from GSM-phones
CD-ROM server needs recalibration
firewall needs cooling
asynchronous inode failure
transient bus protocol violation
incompatible bit-registration operators
your process is not ISO 9000 compliant
You need to upgrade your VESA local bus to a MasterCard local bus.
The recent proliferation of Nuclear Testing
Elves on strike. (Why do they call EMAG Elf Magic)
Internet exceeded Luser level, please wait until a luser logs off before attempting to log back on.
Your EMAIL is now being delivered by the USPS.
Your computer hasn't been returning all the bits it gets from the Internet.
You've been infected by the Telescoping Hubble virus.
Scheduled global CPU outage
Your Pentium has a heating problem - try cooling it with ice cold water.(Do not turn of your computer, you do not want to cool down the Pentium Chip while he isn't working, do you?)
Your processor has processed too many intructions.  Turn it off emideately, do not type any commands!!
Your packets were eaten by the terminator
Your processor does not develop enough heat.
We need a licensed electrician to replace the light bulbs in the computer room.
The POP server is out of Coke
Fiber optics caused gas main leak
Server depressed, needs Prozak
quatnum decoherence
those damn racoons!
suboptimal routing experience
A plumber is needed, the network drain is clogged
50% of the manual is in .pdf readme files
the AA battery in the wallclock sends magnetic interference
the xy axis in the trackball is coordinated with the summer soltice
the butane lighter causes the pincushioning
old inkjet cartridges emanate barium-based fumes
manager in the cable duct
We'll fix that in the next (upgrade, update, patch release, service pack).
HTTPD Error 666 : BOFH was here
HTTPD Error 4004 : very old Intel cpu - insufficient processing power
The ATM board has run out of 10 pound notes.  We are having a whip round to refill it, care to contribute ?
Network failure -  call NBC
Having to manually track the satellite.
Your/our computer(s) had suffered a memory leak, and we are waiting for them to be topped up.
The rubber band broke
We're on Token Ring, and it looks like the token got loose.
Stray Alpha Particles from memory packaging caused Hard Memory Error on Server.
paradigm shift...without a clutch
PEBKAC (Problem Exists Between Keyboard And Chair)
The cables are not the same length.
Second-sytem effect.
Chewing gum on /dev/sd3c
Boredom in the Kernel.
the daemons! the daemons! the terrible daemons!
I'd love to help you -- it's just that the Boss won't let me near the computer. 
struck by the Good Times virus
YOU HAVE AN I/O ERROR -> Incompetent Operator error
Your parity check is overdrawn and you're out of cache.
Communist revolutionaries taking over the server room and demanding all the computers in the building or they shoot the sysadmin. Poor misguided fools.
Plasma conduit breach
Out of cards on drive D:
Sand fleas eating the Internet cables
parallel processors running perpendicular today
ATM cell has no roaming feature turned on, notebooks can't connect
Webmasters kidnapped by evil cult.
Failure to adjust for daylight savings time.
Virus transmitted from computer to sysadmins.
Virus due to computers having unsafe sex.
Incorrectly configured static routes on the corerouters.
Forced to support NT servers; sysadmins quit.
Suspicious pointer corrupted virtual machine
Its the InterNIC's fault.
Root name servers corrupted.
Budget cuts forced us to sell all the power cords for the servers.
Someone hooked the twisted pair wires into the answering machine.
Operators killed by year 2000 bug bite.
We've picked COBOL as the language of choice.
Operators killed when huge stack of backup tapes fell over.
Robotic tape changer mistook operator's tie for a backup tape.
Someone was smoking in the computer room and set off the halon systems.
Your processor has taken a ride to Heaven's Gate on the UFO behind Hale-Bopp's comet.
t's an ID-10-T error
Dyslexics retyping hosts file on servers
The Internet is being scanned for viruses.
Your computer's union contract is set to expire at midnight.
Bad user karma.
/dev/clue was linked to /dev/null
Increased sunspot activity.
We already sent around a notice about that.
It's union rules. There's nothing we can do about it. Sorry.
Interferance from the Van Allen Belt.
Jupiter is aligned with Mars.
Redundant ACLs. 
Mail server hit by UniSpammer.
T-1's congested due to porn traffic to the news server.
Data for intranet got routed through the extranet and landed on the internet.
We are a 100% Microsoft Shop.
We are Microsoft.  What you are experiencing is not a problem; it is an undocumented feature.
Sales staff sold a product we don't offer.
Secretary sent chain letter to all 5000 employees.
Sysadmin didn't hear pager go off due to loud music from bar-room speakers.
Sysadmin accidentally destroyed pager with a large hammer.
Sysadmins unavailable because they are in a meeting talking about why they are unavailable so much.
Bad cafeteria food landed all the sysadmins in the hospital.
Route flapping at the NAP.
Computers under water due to SYN flooding.
The vulcan-death-grip ping has been applied.
Electrical conduits in machine room are melting.
Traffic jam on the Information Superhighway.
Radial Telemetry Infiltration
Cow-tippers tipped a cow onto the server.
tachyon emissions overloading the system
Maintence window broken
We're out of slots on the server
Computer room being moved.  Our systems are down for the weekend.
Sysadmins busy fighting SPAM.
Repeated reboots of the system failed to solve problem
Feature was not beta tested
Domain controler not responding
Someone else stole your IP address, call the Internet detectives!
It's not RFC-822 compliant.
operation failed because: there is no message for this error (#1014)
stop bit received
internet is needed to catch the etherbunny
network down, IP packets delivered via UPS
Firmware update in the coffee machine
Temporal anomaly
Mouse has out-of-cheese-error
Borg implants are failing
Borg nanites have infested the server
error: one bad user found in front of screen
Please state the nature of the technical emergency
Internet shut down due to maintainance
Daemon escaped from pentagram
crop circles in the corn shell
sticky bit has come loose
Hot Java has gone cold
Cache miss - please take better aim next time
Hash table has woodworm
Trojan horse ran out of hay
Zombie processess detected, machine is haunted.
overflow error in /dev/null
Browser's cookie is corrupted -- someone's been nibbling on it.
Mailer-daemon is busy burning your message in hell.
According to Microsoft, it's by design
vi needs to be upgraded to vii
greenpeace free'd the mallocs
Terorists crashed an airplane into the server room, have to remove /bin/laden. (rm -rf /bin/laden)
"""

matrixtxt = """
<morpheus> If real is what you can feel, smell, taste and see, then 'real' is simply electrical signals interpreted by your brain
<morpheus> What are you waiting for? You're faster than this. Don't think you are, know you are. Come on! Stop trying to hit me and hit me!
<Morpheus> Unfortunately, no one can be told what the Matrix is. You have to see it for yourself.
<morpheus> What you know you can't explain, but you feel it. You've felt it your entire life, that there's something wrong with the world. You don't know what it is, but it's there, like a splinter in your mind, driving you mad.
<morpheus> You have the look of a man who accepts what he sees because he is expecting to wake up. Ironically, that's not far from the truth.
<morpheus> I'm trying to free your mind, Neo. But I can only show you the door. You're the one that has to walk through it.
<neo> Why do my eyes hurt? <morpheus> You've never used them before.
<morpheus> There is a difference between knowing the path and walking the path.
<neo> What are you trying to tell me? That I can dodge bullets? <morpheus> No, Neo. I'm trying to tell you that when you're ready, you won't have to.
<morpheus> The Matrix is the world that has been pulled over your eyes to blind you from the truth.
<morpheus> You've been living in a dream world, Neo.
<morpheus> I imagine that right now you're feeling a bit like Alice, tumbling down the rabbit hole.
<morpheus> What is "real"? How do you define "real"?
<morpheus> Welcome to the desert of the real.
<morpheus> You have to let it all go, Neo. Fear, doubt, and disbelief. Free your mind.
<neo> Am I dead? <morpheus> Far from it.
<morpheus> You are the One, Neo. You see, you may have spent the last few years looking for me, but I have spent my entire life looking for you.
<morpheus> This is your last chance. After this, there is no turning back. You take the blue pill - the story ends, you wake up in your bed and believe whatever you want to believe. You take the red pill - you stay in Wonderland and I show you how deep the rabbit-hole goes.
<morpheus> I've seen an agent punched through a concrete wall. Men have emptied entire clips at them and hit nothing but air, yet their strength and their speed are still based in a world that is built on rules. Because of that, they will never be as strong or as fast as you can be. 
<rhineheart> You have a problem with authority, Mr. Anderson.
<tank> Hey Mikey I think he likes it.
<morpheus> It IS our destiny
<agent smith> Never send a human to do a machine's job.
<morpheus> Focus, trinity
<agent smith> Do you hear that, Mr. Anderson? That is the sound of inevitability
<spoon boy> There is no spoon. 
<tank> So what do you need? Besides a miracle <neo> Guns. Lots of guns.
<trinity> Neo... nobody has ever done this before. <neo> I know. That's why it's going to work.
<neo> I know kung fu
<neo> Okey dokey.. free my mind. Right, no problem, free my mind, free my mind, no problem, right...
<cypher> Good shit, huh? It's good for two things: degreasing engines and killing brain cells.
<cypher> All I see now is blonde, brunette, redhead.
"""

motivationtxt = """
Achievement: You can do anything you set your mind to when you have vision, determination, and an endless supply of expendable labor. 
Adversity: That which does not kill me postpones the inevitable. 
Agony: Not all pain is gain. 
Ambition: The journey of a thousand miles sometimes ends very, very badly.
Apathy: If we don't take care of the customer,maybe they'll stop bugging us.
Arrogance: The best leaders inspire by example. When that's not an option, brute intimidation works pretty well, too.
Beauty: If you're attractive enough on the outside, people will forgive you for being irritating to the core.
Bitterness: Never be afraid to share your dreams with the world, because there's nothing the world loves more than the taste of really sweet dreams.
Blame: The secret to success is knowing who to blame for your failures.
Burnout: Attitudes are contagious. Mine might kill you. 
Change: It's a short trip from riding the waves of change to being torn apart by the jaws of defeat. 
Change (winds): When the winds of change blow hard enough, the most trivial of things can become deadly projectiles.
Cluelessness: There are no stupid questions, but there are a LOT of inquisitive idiots.
Compromise: Let's agree to respect each others views, no matter how wrong yours may be.
Conformity: When people are free to do as they please, they usually imitate each other.
Consulting: If you're not a part of the solution,there's good money to be made in prolonging the problem. 
Dare to Slack: When birds fly in the right formation, they need only exert half the effort. Even in nature, teamwork results in collective laziness.
Defeat: For every winner, there are dozens of losers. Odds are you're one of them.
Delusions: There is no greater joy than soaring high on the wings of your dreams, except maybe the joy of watching a dreamer who has nowhere to land but in the ocean of reality. 
Demotivation: Sometimes the best solution to morale problems is just to fire all of the unhappy people. 
Despair: It's always darkest just before it goes pitch black. 
Destiny: You were meant for me. Perhaps as a punishment.
Discovery: A company that will go to the ends of the Earth for its people will find it can hire them for about 10% of the cost of Americans.
Disloyalty: There comes a time when every team must learn to make individual sacrifices.
Disservice: It takes months to find a customer, but only seconds to lose one... the good news is that we should run out of them in no time.
Do it Later: The early worm is for the birds.
Doubt: In the battle between you and the world, bet on the world.
Dreams: Dreams are like rainbows. Only idiots chase them. 
Dysfunction: The only consistent feature in all of your dissatisfying relationships is you.
Effort: Hard work never killed anybody, but it is illegal in some places. 
Elitism: It's lonely at the top, but it's comforting to look down upon everyone at the bottom.
Failure: When your best just isn't good enough. 
Fear: Until you have the courage to lose sight of the shore, you will not know the terror of being forever lost at sea 
Flattery: If you want to get to the top, prepare to kiss a lot of the bottom.
Futility: You'll always miss 100% of the shots you don't take, and, statistically speaking, 99% of the shots you do. 
Get To Work: You aren't being paid to believe in the power of your dreams. 
Goals: It's best to avoid standing directly between a competitive jerk and his goals. 
Hazards: There is an island of opportunity in the middle of every difficulty.  Miss that, though, and you're pretty much doomed. 
Humiliation: The harder you try, the dumber you look. 
Idiocy: Never underestimate the power of stupid people in large groups. 
Ignorance: It's amazing how much easier it is for a team to work together when no one has any idea where they're going. 
Incompetence: When you earnestly believe you can compensate for a lack of skill by doubling your efforts, there's no end to what you can't do. 
Indifference: It takes 43 muscles to frown and 17 to smile, but it doesn't take any to just sit there with a dumb look on your face.
Individuality: Always remember that you are unique. Just like everybody else. 
Ineptitude: If you can't learn to do something well, learn to enjoy doing it poorly.
Insanity: It's difficult to comprehend how insane some people can be. Especially when you're insane. 
Inspiration: Genius is 1 percent inspiration and 99% perspiration, which is why engineers sometimes smell really bad. 
Intimidation: No one can make you feel inferior without your consent, but you'd be a fool to withhold that from your superiors.
Irresponsibility: No single raindrop believes it is to blame for the flood.
Laziness: Success is a journey, not a destination. So stop running. 
Leaders: Leaders are like eagles. We don't have either of them here. 
Limitations: Until you spread your wings, you'll have no idea how far you can walk.
Loneliness: If you find yourself struggling with loneliness, you're not alone. And yet you are alone. So very alone. 
Losing: If at first you don't succeed, failure may be your style.
Madness: Madness does not always howl. Sometimes, it is the quiet voice at the end of the day saying, "Hey, is there room in your head for one more?"
Mediocrity: It takes a lot less time and most people won't notice the difference until it's too late. 
Meetings: None of us is as dumb as all of us. 
Misfortune: While good fortune often eludes you, this kind never misses. 
Mistakes: It could be that the purpose of your life is only to serve as a warning to others.
Motivation: If a pretty poster and a cute saying are all it takes to motivate you, you probably have a very easy job. The kind robots will be doing soon. 
Nepotism: We promote family values here - almost as often as we promote family members. 
Overconfidence: Before you attempt to beat the odds, be sure you could survive the odds beating you. 
Pessimism: Every dark cloud has a silver lining, but lightning kills hundreds of people each year who are trying to find it. 
Persistence: It's over, man. Let her go.
Planning: Much work remains to be done before we can announce our total failure to make any progress. 
Potential: Not everyone gets to be an astronaut when they grow up.
Power: Power corrupts. Absolute power corrupts absolutely. But it rocks absolutely, too.
Pressure: It can turn a lump of coal into a flawless diamond, or an average person into a perfect basketcase.
Pretension: The downside of being better than everyone else is that people tend to assume you're pretentious. 
Problems: No matter how great and destructive your problems may seem now, remember, you've probably only seen the tip of them 
Procrastination: Hard work often pays off after time, but laziness always pays off now. 
Quality: The race for quality has no finish line- so technically, it's more like a death march. 
Regret: It hurts to admit when you make mistakes - but when they're big enough, the pain only lasts a second 
Retirement: Because you've given so much of yourself to the Company that you don't have anything left we can use. 
Risks: If you never try anything new, you'll miss out on many of life's great disappointments.
Sacrifice: Your role may be thankless, but if you're willing to give it your all, you just might bring success to those who outlast you.
Sacrifice (Temple): All we ask here is that you give us your heart. 
Strife: As long as we have each other, we'll never run out of problems. 
Stupidity: Quitters never win, winners never quit, but those who never win AND never quit are idiots.
Success: Some people dream of success, while other people live to crush those dreams. 
Teamwork: A few harmless flakes working together can unleash an avalanche of destruction.
Trouble: Luck can't last a lifetime unless you die young. 
Underachievement: The tallest blade of grass is the first to be cut by the lawnmower. 
Wishes: When you wish upon a falling star, your dreams can come true. Unless it's really a meteorite hurtling to the Earth which will destroy all life. Then you're pretty much hosed no matter what you wish for. Unless it's death by meteor. 
Worth: Just because you're necessary doesn't mean you're important.
"""



# gozerplugs/trac.py
#
#

from gozerbot.commands import cmnds
from gozerbot.examples import examples
from gozerbot.aliases import aliasset
from gozerbot.persist.persistconfig import PersistConfig

cfg = PersistConfig()
cfg.define('baseurl', 'http://dev.gozerbot.org')

def handle_tracwiki(bot, ievent):
    if not ievent.rest:
        ievent.missing("<item>")
        return
    ievent.reply('%s/wiki/%s' % (cfg.get('baseurl'), ievent.rest))

cmnds.add('trac-wiki', handle_tracwiki, 'USER')
examples.add('trac-wiki', 'give t.e.o wiki url', 'trac-wiki TracAdmin')
aliasset('wiki', 'trac-wiki')



# plugs/oneliner.py
#
#
#

""" uses the random lib """

__copyright__ = 'this file is in the public domain'
__revision__ = '$Id: oneliner.py 517 2006-12-21 05:00:00Z deck $'

from gozerbot.generic import handle_exception
from gozerbot.commands import cmnds
from gozerbot.examples import examples
from gozerbot.plughelp import plughelp
import re, random

plughelp.add('oneliner', 'display a oneliner')

oneliners=["100,000 sperm and you were the fastest?",
	"42.7 percent of all statistics are made up on the spot.",
	"99 percent of lawyers give the rest a bad name.",
	"A bartender is just a pharmacist with a limited inventory.",
	"A clean desk is a sign of a cluttered desk drawer.",
	"A clear conscience is usually the sign of a bad memory.",
	"A closed mouth gathers no foot.",
	"A conclusion is the place where you got tired of thinking.",
	"A day without sunshine is like, night.",
	"A diplomat is someone who can tell you to go to hell in such a way that you will look forward to the trip.",
	"A flashlight is a case for holding dead batteries.",
	"All generalizations are false, including this one.",
	"All men are idiots, and I married their King.",
	"Always remember you're unique, just like everyone else.",
	"Always try to be modest and be proud of it!",
	"Anything worth taking seriously is worth making fun of.",
	"Artificial Intelligence usually beats real stupidity.",
	"Assassins do it from behind.",
	"Atheism is a non-prophet organization.",
	"Auntie Em, Hate you, hate Kansas, taking the dog. Dorothy.",
	"Be nice to your kids. They'll choose your nursing home.",
	"Beauty is in the eye of the beer holder.",
	"Beer: It's not just for breakfast anymore.",
	"Before you criticize someone, you should walk a mile in their shoes. That way, when you criticize them, you're a mile away and you have their shoes.",
	"Better to understand a little than to misunderstand a lot.",
	"Bills travel through the mail at twice the speed of checks",
	"Borrow money from a pessimist, they don't expect it back.",
	"Boycott shampoo! Demand the REAL poo!",
	"C program run. C program crash. C programmer quit.",
	"Change is inevitable, except from a vending machine.",
	"Chocolate: the OTHER major food group.",
	"Consciousness: That annoying time between naps.",
	"Corduroy pillows: They're making headlines!",
	"Could you drive any better if I shoved that cell phone up your ass?",
	"Criminal Lawyer is a redundancy.",
	"Daddy, why doesn't this magnet pick up this floppy disk?",
	"Death is hereditary.",
	"Despite the cost of living, have you noticed how popular it remains?",
	"Did anyone see my lost carrier?",
	"Diplomacy is the art of saying good doggie while looking for a bigger stick.",
	"Do not walk behind me, for I may not lead. Do not walk ahead of me, for I may not follow. Do not walk beside me, either. Just leave me alone.",
	"Don't be irreplaceable; if you can't be replaced, you can't be promoted.",
	"Don't drink and drive. You might hit a bump and spill your drink.",
	"Don't piss me off! I'm running out of places to hide the bodies.",
	"Don't take life too seriously, you won't get out alive.",
	"Double your drive space. Delete Windows!",
	"Duct tape is like the force, it has a light side and a dark side and it holds the universe together.",
	"Eagles may soar, but weasels don't get sucked into jet engines.",
	"Energizer Bunny arrested and charged with battery.",
	"Error, no keyboard. Press F1 to continue.",
	"Ever notice how fast Windows runs? Neither did I.",
	"Ever stop to think, and forget to start again?",
	"Everyone has a photographic memory. Some don't have film.",
	"Experience is something you don't get until just after you need it.",
	"Experience is what you get when you didn't get what you wanted.",
	"Few women admit their age. Few men act theirs.",
	"For every action there is an equal and opposite criticism.",
	"For Sale: Parachute. Only used once, never opened, small stain.",
	"Forget world peace. Visualize using your turn signal.",
	"Friends help you move. Real friends help you move bodies.",
	"Friends may come and go, but enemies tend to accumulate.",
	"Generally speaking, you aren't learning much when your mouth is moving.",
	"Genius does what it must, talent does what it can, and you had best do what you're told.",
	"Get a new car for your spouse; it'll be a great trade!",
	"Give a man a fish and he will eat for a day. Teach him how to fish, and he will sit in a boat and drink beer all day.",
	"Give me ambiguity or give me something else.",
	"Good judgment comes from bad experience and a lot of that comes from bad judgment.",
	"Hard work has a future payoff. Laziness pays off now.",
	"He who laughs last thinks slowest.",
	"Honk if you love peace and quiet.",
	"Honk if you want to see my finger.",
	"How do you tell when you run out of invisible ink?",
	"How does Teflon stick to the pan?",
	"How many of you believe in telekinesis? Raise my hand.",
	"I am not a vegetarian because I love animals. I am a vegetarian because I hate plants.",
	"I couldn't repair your brakes, so I made your horn louder.",
	"I didn't say it was your fault. I said I was going to blame you.",
	"I don't suffer from insanity. I enjoy every minute of it.",
	"I feel like I'm diagonally parked in a parallel universe.",
	"I get enough exercise just pushing my luck.",
	"I just got lost in thought. It was unfamiliar territory.",
	"I need someone really bad. Are you really bad?",
	"I poured Spot remover on my dog. Now he's gone.",
	"I took an IQ test and the results were negative.",
	"I tried sniffing Coke once, but the ice cubes got stuck in my nose.",
	"I used to be indecisive. Now I'm not sure.",
	"I used to have a handle on life, and then it broke.",
	"I used to have an open mind but my brains kept falling out.",
	"I won't rise to the occasion, but I'll slide over to it.",
	"I wouldn't be caught dead with a necrophiliac.",
	"I'm as confused as a baby in a topless bar.",
	"I'm not a complete idiot, some parts are missing!",
	"I'm writing a book. I've got the page numbers done.",
	"If at first you don't succeed, destroy all evidence that you tried.",
	"If at first you don't succeed, skydiving is not for you.",
	"If Barbie is so popular, why do you have to buy her friends?",
	"If ignorance is bliss, you must be orgasmic.",
	"If you can read this, I can slam on my brakes and sue you!",
	"If you can't convince them, confuse them.",
	"If you choke a smurf, what color does it turn?",
	"If you get to it and you can't do it, well there you jolly well are, aren't you.",
	"If you haven't much education you must use your brain.",
	"If you lend someone $20, and never see that person again; it was probably worth it.",
	"If you tell the truth you don't have to remember anything.",
	"If you think nobody cares if you're alive, try missing a couple of car payments.",
	"IRS: We've got what it takes to take what you've got.",
	"It IS as bad as you think, and they ARE out to get you.",
	"It is far more impressive when others discover your good qualities without your help.",
	"It may be that your sole purpose in life is simply to serve as a warning to others.",
	"It's always darkest before dawn. So if you're going to steal the neighbor's newspaper, that's the time to do it.",
	"It's lonely at the top, but you eat better.",
	"Jack Kevorkian for White House Physician.",
	"Jesus loves you, but everyone else thinks you're an asshole.",
	"Join the Army, meet interesting people, and kill them.",
	"Keep honking. I'm reloading.",
	"Laughing stock: cattle with a sense of humor.",
	"Learn from your parents' mistakes: use birth control.",
	"Lottery: A tax on people who are bad at math.",
	"Love may be blind but marriage is a real eye-opener.",
	"Make it idiot proof and someone will make a better idiot.",
	"Monday is an awful way to spend 1/7th of your life.",
	"Montana: At least our cows are sane!",
	"More hay, Trigger? No thanks, Roy, I'm stuffed!",
	"Multitasking means screwing up several things at once.",
	"My hockey mom can beat up your soccer mom.",
	"My mind is like a steel trap, rusty and illegal in 37 states.",
	"Never ask a barber if he thinks you need a haircut.",
	"Never do card tricks for the group you play poker with.",
	"Never mess up an apology with an excuse.",
	"Never miss a good chance to shut up.",
	"Never test the depth of the water with both feet.",
	"Never underestimate the power of stupid people in large groups.",
	"No one is listening until you make a mistake.",
	"Oh Lord, give me patience, and GIVE IT TO ME NOW!",
	"Okay, who put a stop payment on my reality check?",
	"On the other hand, you have different fingers.",
	"Oops. My brain just hit a bad sector.",
	"Out of my mind. Back in five minutes.",
	"Pentiums melt in your PC, not in your hand.",
	"Plan to be spontaneous, tomorrow.",
	"Pride is what we have. Vanity is what others have.",
	"Puritanism: The haunting fear that someone, somewhere may be happy.",
	"Quantum mechanics: The dreams stuff is made of.",
	"Quickly, I must hurry, for there go my people and I am their leader.",
	"Reality is a crutch for people who can't handle drugs.",
	"Remember half the people you know are below average.",
	"Save the whales. Collect the whole set",
	"Save your breath. You'll need it to blow up your date!",
	"Sex is like air; it's not important unless you aren't getting any.",
	"Shin: a device for finding furniture in the dark.",
	"Smile, it's the second best thing you can do with your lips.",
	"Smith & Wesson: The original point and click interface.",
	"Some days you are the bug, some days you are the windshield.",
	"Some drink at the fountain of knowledge. Others just gargle.",
	"Some people are only alive because it is illegal to shoot them.",
	"Success always occurs in private and failure in full view.",
	"Suicidal twin kills sister by mistake!",
	"Support bacteria, they're the only culture some people have.",
	"The Bermuda Triangle got tired of warm weather. It moved to Finland. Now Santa Claus is missing.",
	"The colder the x-ray table, the more of your body is required on it.",
	"The early bird may get the worm, but the second mouse gets the cheese.",
	"The hardness of butter is directly proportional to the softness of the bread.",
	"The journey of a thousand miles begins with a broken fan belt and a flat tire.",
	"The more you complain, the longer God makes you live.",
	"The only substitute for good manners is fast reflexes.",
	"The problem with the gene pool is that there is no lifeguard.",
	"The quickest way to double your money is to fold it in half and put it back in your pocket.",
	"The secret of the universe is @*&^^^ NO CARRIER",
	"The severity of the itch is inversely proportional to the ability to reach it.",
	"The sex was so good that even the neighbors had a cigarette.",
	"The shortest distance between two points is under construction.",
	"The sooner you fall behind the more time you'll have to catch up.",
	"The universe is a figment of its own imagination. There's no future in time travel.",
	"There are 3 kinds of people: those who can count & those who can't.",
	"There are two theories to arguing with women. Neither one works.",
	"There's too much blood in my caffeine system.",
	"Things are more like they are now than they ever were before.",
	"Time is the best teacher; unfortunately it kills all of its students.",
	"Time is what keeps everything from happening at once.",
	"Timing has an awful lot to do with the outcome of a rain dance.",
	"To steal ideas from one person is plagiarism; to steal from many is research.",
	"To succeed in politics, it is often necessary to rise above your principles.",
	"Very funny Scotty, now beam down my clothes.",
	"Wanted: Meaningful overnight relationship.",
	"Warning: Dates in calendar are closer than they appear.",
	"We have enough youth, how about a fountain of smart?",
	"We were born naked, wet and hungry. Then things got worse.",
	"Wear short sleeves! Support your right to bare arms!",
	"What happens if you get scared half to death twice?",
	"What is a free gift? Aren't all gifts free?",
	"What's the speed of dark?",
	"When everything's coming your way, you're in the wrong lane and going the wrong way.",
	"When there's a will, I want to be in it.",
	"When you don't know what you are doing, do it neatly.",
	"Who is General Failure and why is he reading my hard disk?",
	"Who stopped payment on my reality check?",
	"Why do psychics have to ask you for your name?",
	"Why is abbreviation such a long word?",
	"Why isn't phonetic spelled the way it sounds?",
	"Women who seek to be equal to men lack ambition.",
	"You are depriving some poor village of its idiot.",
	"You can do more with a kind word and a gun than with just a kind word.",
	"You have the right to remain silent. Anything you say will be misquoted then used against you.",
	"You're just jealous because the voices are talking to me and not you!",
	"Your gene pool could use a little chlorine.",
	"Your kid may be an honors student, but you're still an idiot."]

def handle_oneliner(bot, ievent):
    data = oneliners[random.randint(0,len(oneliners)-1)]
    ievent.reply(data)

cmnds.add('oneliner', handle_oneliner, 'USER')
examples.add('oneliner', 'show oneliner', 'oneliner')



# Yahoo! search
# (c) Wijnand 'tehmaze' Modderman - http://tehmaze.com
# (c) Leif Hedstrom <leif@ogre.com>, creator of pYsearch (License: BSD)

__author__ = 'Wijnand Modderman <gozerbot@tehmaze.com>'
__copyright__ = 'BSD'

import time
import urllib
import xml.dom.minidom
from gozerbot.aliases import aliases
from gozerbot.commands import cmnds
from gozerbot.generic import geturl, waitforqueue
from gozerbot.persist.persistconfig import PersistConfig
from gozerbot.plughelp import plughelp

plughelp.add('yahoo', 'query yahoo search engine')

cfg = PersistConfig()
cfg.define('appid', '')

# Function: string_to_bool
#   Convert a string argument to a boolean
#
# Parmeters:
#   s - The input string
#
# Returns:
#   bool
def string_to_bool(s):
    sb = {'false': False, 'true': True}
    if sb.has_key(s.lower()):
        return sb[s.lower()]
    return bool(s)

class YahooException(Exception):
    pass

class YahooError(Exception):
    pass

class YahooXMLError(Exception):
    pass

class YahooResultDict(dict):
    def __getattr__(self, key):
        try:
            return self[key]
        except KeyError:
            raise AttributeError("Result object has no attribute '%s'" % key)

# Class: YahooResultParser
#   Parse the results found by a <YahooResultParser> 
class YahooResultParser(object):
    # Constructor: __init__
    # Parameters:
    #   service - ...
    #   res_dict - ...
    def __init__(self, service, res_dict=YahooResultDict):
        self._service = service
        self._total_results_available = 0
        self._total_results_returned = 0
        self._first_result_position = 0
        self._results = []
        self._res_dict = res_dict
        self._res_fields = []
        self._init_res_fields()

    def __iter__(self):
        return iter(self._results)

    def _init_res_fields(self):
        """Initialize the valid result fields."""
        self._res_fields = [('Title', None, None),
                            ('Summary', None, None),
                            ('Url', None, None),
                            ('ClickUrl', None, None)]

    def _get_results(self):
        """Get the results."""
        return self._results
    results = property(_get_results, None, None,
                       "The list of all results")

    def _get_service(self):
        """Get the service for this DOM parser."""
        return self._service
    def _set_service(self, service):
        """Set the service for this DOM parser."""
        self._service = service
    service = property(_get_service, _set_service, None,
                       "The Search Web Service object for this results parser")

    def parse_results(self, result_set):
        """Parse the results."""
        err = "Search Result class %s must implement a parse_result()" % (
            self._service.svc_name)
        raise ClassError(err)

    def _get_total_results_available(self):
        """Get the total number of results for the query."""
        return self._total_results_available

    def _get_total_results_returned(self):
        """Get the number of results returned."""
        return self._total_results_returned
    total_results_returned = property(_get_total_results_returned, None, None,
                                      "The number of results returned")
    totalResultsReturned = property(_get_total_results_returned, None, None,
                                    "The number of results returned")

    def _get_first_result_position(self):
        """Get the first result position."""
        return self._first_result_position
    first_result_position = property(_get_first_result_position, None, None,
                                     "The first result position")
    firstResultPosition = property(_get_first_result_position, None, None,
                                   "The first result position") 

class YahooDOMResultParser(YahooResultParser):
    """DomResultParser - Base class for Yahoo Search DOM result parsers

    This is a DOM specific parser that is used as a base class for all
    Yahoo Search result parsers. It obviously must implement the main entry
    entry point, parse_results().
    """
    def parse_results(self, dom_object):
        """This is a simple DOM parser for all Yahoo Search services. It
        expects to find a top-level node named ResultSet. This is the main
        entry point for the DOM parser, and it requires a properly con-
        structed DOM object (e.g. using minidom).
        """
        try:
            result_set = dom_object.getElementsByTagName('ResultSet')[0]
        except:
            raise YahooXMLError("DOM object has no ResultSet")
        self._parse_result_set(result_set)

    def _get_text(self, nodelist, casting=None):
        """Find all text nodes for the nodelist, and concatenate them
        into one resulting strings. This is a helper method for the
        DOM parser.
        """
        rcode = ""
        for node in nodelist:
            if node.nodeType == node.TEXT_NODE:
                rcode = rcode + node.data
        if casting is not None:
            if rcode == "":
                return rcode
            else:
                return casting(rcode)
        else:
            return rcode

    def _tag_to_list(self, node, tag, casting=None):
        """Turn a number of tag elements into a list of values."""
        ret = []
        if casting is not None:
            for item in node.getElementsByTagName(tag):
                ret.append(casting(self._get_text(item.childNodes)))
        else:
            for item in node.getElementsByTagName(tag):
                ret.append(self._get_text(item.childNodes))

    def _tags_to_dict(self, node, tags):
        """Internal method to parse and extract a list of tags from a
        particular node. We return a dict, which can potentially be empty.
        The tags argument is a list of lists, where each sub-list is

            (tag-name, default value/None, casting function/None)

        The default "type" of a value is string, so there is no reason
        to explicitly cast to a str.
        """
        res = self._res_dict()
        for tag in tags:
            elem = node.getElementsByTagName(tag[0])
            if elem:
                val = self._get_text(elem[0].childNodes, tag[2])
            elif tag[1] is not None:
                val = tag[1]
            else:
                raise parser.XMLError("Result is missing a %s node" % tag[0])
            res[tag[0]] = val
        return res

    def _id_attribute_to_dict(self, node):
        """Internal method to parse and extract a node value, which
        has an "id" attribute as well. This will return a result dict
        with two values:

            { 'Name' :  <node-text>, 'Id' : <id attribute> }
        """
        res = self._res_dict()
        res['Name'] = self._get_text(node.childNodes)
        node_id = node.attributes.getNamedItem('id')
        if node_id:
            res['Id'] = str(node_id.nodeValue)
        else:
            raise parser.XMLError("%s node has no id attribute" % node.nodeName)
        return res

    def _parse_list_node(self, node, tag):
        """Internal method to parse a result node, which contains one
        or more data nodes. Each such node is converted to a dict (see
        _id_attribute_to_dict), and we return a list of such dicts.
        """
        res = []
        for elem in node.getElementsByTagName(tag):
            res.append(self._id_attribute_to_dict(elem))
        return res

    def _parse_result_set(self, result_set):
        """Internal method to parse a ResultSet node"""
        attributes = result_set.attributes
        if not attributes:
            raise parser.XMLError("ResultSet has no attributes")

        attr = attributes.getNamedItem('totalResultsAvailable')
        if attr:
            self._total_results_available = int(attr.nodeValue)
        else:
            raise parser.XMLError("ResultSet has no totalResultsAvailable attr")
        attr = attributes.getNamedItem('totalResultsReturned')
        if attr:
            self._total_results_returned = int(attr.nodeValue)
        else:
            raise parser.XMLError("ResultSet has no totalResultsReturned attr")
        attr = attributes.getNamedItem('firstResultPosition')
        if attr:
            self._first_result_position = int(attr.nodeValue)
        else:
            raise parser.XMLError("ResultSet has no firstRestultPosition attr")
        for res in result_set.getElementsByTagName('Result'):
            self._results.append(self._parse_result(res))

    def _parse_result(self, result):
        """Internal method to parse one Result node"""
        return self._tags_to_dict(result, self._res_fields)

#
# The actual parsers
#

class YahooQueryImageSearch(YahooDOMResultParser):
    """ImageSearch - DOM parser for Image Search

    Each result is a dictionary populated with the extracted data from the
    XML results. The following keys are always available:

        Title            - The title of the image file.
        Summary          - Summary text associated with the image file.
        Url              - The URL for the image file or stream.
        ClickUrl         - The URL for linking to the image file.
        RefererUrl       - The URL of the web page hosting the content.
        FileSize         - The size of the file, in bytes.
        FileFormat       - One of bmp, gif, jpg or png.
        Thumbnail        - The URL of the thumbnail file.

    The following attributes are optional, and might not be set:

        Height           - The height of the image in pixels.
        Width            - The width of the image in pixels.
        Publisher        - The creator of the image file.
        Restrictions     - Provides any restrictions for this media
                           object. Restrictions include noframe and
                           noinline.
        Copyright        - The copyright owner.

    The Thumbnail is in turn another dictionary, which will have the
    following keys:

        Url             - URL of the thumbnail.
        Copyright        - The copyright owner.

    The Thumbnail is in turn another dictionary, which will have the
    following keys:

        Url             - URL of the thumbnail.
        Height          - Height of the thumbnail, in pixels (optional).
        Width           - Width of the thumbnail, in pixels (optional).


    Example:
        results = ws.parse_results(dom)
        for res in results:
            print "%s - %s bytes" % (res.Title, res.FileSize)
    """
    def _init_res_fields(self):
        """Initialize the valid result fields."""
        super(YahooQueryImageSearch, self)._init_res_fields()
        self._res_fields.extend((('RefererUrl', None, None),
                                 ('FileSize', None, int),
                                 ('FileFormat', None, None),
                                 ('Height', 0, int),
                                 ('Width', 0, int),
                                 ('Publisher', "", None),
                                 ('Restrictions', "", None),
                                 ('Copyright', "", None)))

    def _parse_result(self, result):
        """Internal method to parse one Result node"""
        res = super(YahooQueryImageSearch, self)._parse_result(result)
        node = result.getElementsByTagName('Thumbnail')
        if node:
            res['Thumbnail'] = self._tags_to_dict(node[0], (('Url', None, None),
                                                            ('Height', 0, int),
                                                            ('Width', 0, int)))
        else:
            raise parser.XMLError("ImageSearch DOM object has no Thumbnail")
        return res

class YahooQueryPageData(YahooDOMResultParser):
    """PageData - DOM parser for PageData results

        Title            - The title of the web page.
        Url              - The URL for the web page.
        ClickUrl         - The URL for linking to the page.

    """
    def _init_res_fields(self):
        """Initialize the valid result fields."""
        self._res_fields = [('Title', None, None),
                            ('Url', None, None),
                            ('ClickUrl', None, None)]

class YahooQueryWebSearch(YahooDOMResultParser):
    """WebSearch 

    Each result is a dictionary populated with the extracted data from the
    XML results. The following keys are always available:

        Title            - The title of the web page.
        Summary          - Summary text associated with the web page.
        Url              - The URL for the web page.
        ClickUrl         - The URL for linking to the page.

    The following attributes are optional, and might not be set:

        ModificationDate - The date the page was last modified, Unix time.
        MimeType         - The MIME type of the page.
        Cache            - The URL of the cached result, and its size.

    If present, the Cache value is in turn another dictionary, which will
    have the following keys:

        Url             - URL to cached data.
        Size            - Size of the cached entry, in bytes.
    """
    def _init_res_fields(self):
        """Initialize the valid result fields."""
        super(YahooQueryWebSearch, self)._init_res_fields()
        self._res_fields.extend((('ModificationDate', "", None),
                                 ('MimeType', "", None)))

    def _parse_result(self, result):
        """Internal method to parse one Result node"""
        res = super(YahooQueryWebSearch, self)._parse_result(result)
        node = result.getElementsByTagName('Cache')
        if node:
            res['Cache'] = self._tags_to_dict(node[0], (('Url', None, None),
                                                        ('Size', None, None)))
        else:
            res['Cache'] = None
        return res

class YahooQueryVideoSearch(YahooDOMResultParser):
    """VideoSearch - DOM parser for Video Search

    Each result is a dictionary populated with the extracted data from the
    XML results. The following keys are always available:

        Title            - The title of the video file.
        Summary          - Summary text associated with the video file.
        Url              - The URL for the video file or stream.
        ClickUrl         - The URL for linking to the video file.
        RefererUrl       - The URL of the web page hosting the content.
        FileSize         - The size of the file, in bytes.
        FileFormat       - One of avi, flash, mpeg, msmedia, quicktime
                           or realmedia.
        Duration         - The duration of the video file in seconds.
        Streaming        - Whether the video file is streaming or not.

    The following attributes are optional, and might not be set:

        Height           - The height of the keyframe Yahoo! extracted
                           from the video, in pixels.
        Width            - The width of the keyframe Yahoo! extracted
                           from the video, in pixels.
        Channels         - Channels in the audio stream.
        Thumbnail        - The URL of the thumbnail file.
        Publisher        - The creator of the video file.
        Restrictions     - Provides any restrictions for this media
                           object. Restrictions include noframe and
                           noinline.
        Copyright        - The copyright owner.

    If present, the Thumbnail value is in turn another dictionary, which will
    have these keys:

        Url             - URL of the thumbnail.
        Height          - Height of the thumbnail in pixels (optional).
        Width           - Width of the thumbnail in pixels (optional).


    Example:
        results = ws.parse_results(dom)
        for res in results:
            print "%s - %s bytes" % (res.Title, res.FileSize)
    """
    def _init_res_fields(self):
        """Initialize the valid result fields."""
        super(YahooQueryVideoSearch, self)._init_res_fields()
        self._res_fields.extend((('RefererUrl', None, None),
                                 ('FileSize', None, int),
                                 ('FileFormat', None, str),
                                 ('Height', 0, int),
                                 ('Width', 0, int),
                                 ('Streaming', None, string_to_bool),
                                 ('Duration', None, float),
                                 ('Channels', "", str),
                                 ('Publisher', "", None),
                                 ('Restrictions', "", str),
                                 ('Copyright', "", None)))

    def _parse_result(self, result):
        """Internal method to parse one Result node"""
        res = super(YahooQueryVideoSearch, self)._parse_result(result)
        node = result.getElementsByTagName('Thumbnail')
        if node:
            res['Thumbnail'] = self._tags_to_dict(node[0], (('Url', None, None),
                                                            ('Height', 0, int),
                                                            ('Width', 0, int)))
        else:
            res['Thumbnail'] = None
        return res

# Class: Yahoo
#   Generic interface to all YahooQuerySearch classes
class Yahoo:

    # Variable: meta
    #   Information about lookup methods, their YahooQuerySearch class and the base URL
    meta = {
        'ImageSearch': [YahooQueryImageSearch, 'http://search.yahooapis.com/ImageSearchService/V1/imageSearch'],  
        'PageData':    [YahooQueryPageData, 'http://search.yahooapis.com/SiteExplorerService/V1/pageData'],
        'VideoSearch': [YahooQueryVideoSearch, 'http://search.yahooapis.com/VideoSearchService/V1/videoSearch'],  
        'WebSearch':   [YahooQueryWebSearch, 'http://search.yahooapis.com/WebSearchService/V1/webSearch']
    }
    parser = None

    def _get(self, app, parameters):
        url  = self._url(app, parameters)
        data = geturl(url)
        return self._parse(app, data)

    def _parse(self, app, data):
        if not self.parser:
            self.parser = xml.dom.minidom.parseString
        xmlobj = self.parser(data)
        parser = self.meta[app][0](app)
        parser.parse_results(xmlobj)
        return parser

    def _url(self, app, parameters={}):
        if not self.meta.has_key(app):
            raise YahooException('Invalid service "%s"' % app)
        if not cfg.get('appid'):
            raise YahooException('No appid configured, see http://developer.yahoo.com/search/')
        parameters['appid'] = cfg.get('appid')
        return '%s?%s' % (self.meta[app][1], urllib.urlencode(parameters))

    # Function: imageSearch
    #   Search for an image by keyword on the web
    #
    # Parameters:
    #   query - keywords to search for
    #   format - image format to filter on
    #   adult_ok - include adult content in the search result
    #   results - limit the number of results
    #
    # Returns:
    #   results
    def imageSearch(self, query, format='any', adult_ok=0, results=10):
        return self._get('ImageSearch', {'query': query, 'format': format, 'results': results, 'adult_ok': adult_ok})

    def pageData(self, website, results=10):
        return self._get('PageData', {'query': website, 'results': results})

    # Function: webSearch
    #   Search for a keyword on the web
    #
    # Parameters:
    #   query - keywords to search for
    #   results - limit the number of results
    #
    # Returns:
    #   results
    def webSearch(self, query, results=10):
        return self._get('WebSearch', {'query': query, 'results': results})

    # Function: videoSearch
    #   Search for an video by keyword on the web
    #
    # Parameters:
    #   query - keywords to search for
    #   format - video format to filter on
    #   adult_ok - include adult content in the search result
    #   results - limit the number of results
    #
    # Returns:
    #   results
    def videoSearch(self, query, format='any', adult_ok=0, results=10):
        return self._get('VideoSearch', {'query': query, 'format': format, 'results': results, 'adult_ok': adult_ok})

yahoo = Yahoo()

def handle_yahoo_image(bot, ievent):
    adult_ok = 0
    format = ''
    formats = ['any', 'all', 'bmp', 'gif', 'jpeg', 'png']
    args = ievent.args
    try:
        if '--adult' in args:
            adult_ok = 1
            args.remove('--adult')
        if '--format' in args:
            format = args.pop(args.index('--format')+1)
            args.remove('--format')
    except ValueError:
        ievent.missing('[--adult] [--format [<format>] <query>')
        return
    if not ievent.args:
        ievent.missing('[--adult] [--format [<format>] <query>')
        return
    if format and not format in formats:
        ievent.reply('invalid format, available formats: %s' % ', '.join(formats))
        return
    try:
        search = yahoo.imageSearch(' '.join(args), format, adult_ok)
    except YahooException, e:
        ievent.reply(str(e))
        return
    if search._total_results_returned > 0:
        ievent.reply('search for %s ==> %d results available, showing 1 - %d' % \
            (' '.join(args), search._total_results_available,
            search._total_results_returned))
        reply = []
        for result in search._get_results():
             reply.append('%d) %s - %s' % (len(reply)+1, result.Title, result.Url))
        ievent.reply(', '.join(reply))
    else:
        ievent.reply('search for %s ==> no results' % ' '.join(args))

cmnds.add('yahoo-image', handle_yahoo_image, 'USER')

def handle_yahoo_search(bot, ievent):
    if ievent.inqueue:
        text = ' '.join(waitforqueue(ievent.inqueue, 5))
    elif not ievent.args:
        ievent.missing('<query>')
        return
    else:
        text = ' '.join(ievent.args)
    try:
        search = yahoo.webSearch(text)
    except YahooException, e:
        ievent.reply(str(e))
        return
    if search._total_results_returned > 0:
        ievent.reply('search for %s ==> %d results available, showing 1 - %d' % \
            (text, search._total_results_available,
            search._total_results_returned))
        reply = []
        for result in search._get_results():
             reply.append('%d) %s - %s' % (len(reply)+1, result.Title, result.Url))
        ievent.reply(', '.join(reply))
    else:
        ievent.reply('search for %s ==> no results' % ' '.join(ievent.args))

cmnds.add('yahoo-search', handle_yahoo_search, 'USER')
aliases.data['y!'] = 'yahoo-search'
aliases.data['yahoo'] = 'yahoo-search'

def handle_yahoo_pagedata(bot, ievent):
    if not ievent.args:
        ievent.missing('<url>')
        return
    try:
        search = yahoo.pageData(' '.join(ievent.args))
    except YahooException, e:
        ievent.reply(str(e))
        return
    if search._total_results_returned > 0:
        reply = []
        for result in search._get_results():
            reply.append('%d) %s, %dx%d px - %s' % (len(reply)+1, result.Title, 
            result.Width, result.Height, result.Url))
        ievent.reply(', '.join(reply))
    else:
        ievent.reply('pagedata for %s ==> no results' % ' '.join(ievent.args))

cmnds.add('yahoo-pagedata', handle_yahoo_pagedata, ['USER'])


def handle_yahoo_video(bot, ievent):
    adult_ok = 0
    format = ''
    formats = ['any', 'all', 'avi', 'flash', 'mpeg', 'msmedia', 'quicktime', 'realmedia']
    args = ievent.args
    try:
        if '--adult' in args:
            adult_ok = 1
            args.remove('--adult')
        if '--format' in args:
            format = args.pop(args.index('--format')+1)
            args.remove('--format')
    except ValueError:
        ievent.missing('[--adult] [--format [<format>] <query>')
        return
    if not ievent.args:
        ievent.missing('[--adult] [--format [<format>] <query>')
        return
    if format and not format in formats:
        ievent.reply('invalid format, available formats: %s' % ', '.join(formats))
        return
    try:
        search = yahoo.videoSearch(' '.join(args), format, adult_ok)
    except YahooException, e:
        ievent.reply(str(e))
        return
    if search._total_results_returned > 0:
        ievent.reply('search for %s ==> %d results available, showing 1 - %d' % \
            (' '.join(args), search._total_results_available,
            search._total_results_returned))
        reply = []
        for result in search._get_results():
             reply.append('%d) %s, %dx%d, %s - %s' % (len(reply)+1, result.Title, 
             result.Width, result.Height, time.strftime('%T', time.localtime(result.Duration)), result.Url))
        ievent.reply(', '.join(reply))
    else:
        ievent.reply('search for %s ==> no results' % ' '.join(args))

cmnds.add('yahoo-video', handle_yahoo_video, 'USER')




# gozerplugs/beats.py
#
#

""" internet time .beats """

__copyright__ = 'this file is in the public domain'

from gozerbot.commands import cmnds
from gozerbot.examples import examples
from gozerbot.plughelp import plughelp
from gozerbot.plughelp import plughelp
from gozerbot.tests import tests
import time, math

plughelp.add('beats', 'show internet time')

def handle_beats(bot, ievent):
    """ beats .. show current internet time """
    beats = ((time.time() + 3600) % 86400) / 86.4
    beats = int(math.floor(beats))
    ievent.reply('@' + str(beats))

cmnds.add('beats', handle_beats, 'USER')
examples.add('beats', 'show current internet time', 'beats')
tests.add('beats' , '@')



# plugs/alarm.py
#
#

""" the alarm plugin allows for alarms that message the user giving the
 command at a certain time or number of seconds from now
"""

__copyright__ = 'this file is in the public domain'
__depending__ = ['todo', ]

## IMPORT SECTION

# gozerbot imports

from gozerbot.generic import handle_exception, rlog, strtotime, striptime, jsonstring
from gozerbot.utils.lazydict import LazyDict
from gozerbot.persist.persist import Persist
from gozerbot.commands import cmnds
from gozerbot.examples import examples
from gozerbot.utils.nextid import nextid
from gozerbot.utils.locking import lockdec
from gozerbot.fleet import fleet
from gozerbot.datadir import datadir
from gozerbot.plughelp import plughelp
from gozerbot.aliases import aliases
from gozerbot.periodical import periodical
from gozerbot.tests import tests


# basic imports

import time, os, shutil, thread

## END IMPORT

plughelp.add('alarm', 'remind the user with given txt at a certain time')

## LOCK SECTION

alarmlock = thread.allocate_lock()
alarmlocked = lockdec(alarmlock)

# ============
# UPGRADE PART

def upgrade():
    pass

# END UPGRADE
# ===========

class Alarmitem(LazyDict):

    """ item holding alarm data """

    def __init__(self, botname='default', i=0, nick="", ttime=time.time(), txt="", \
printto=None, d={}):
        if not d:
            LazyDict.__init__(self)
        else:
            LazyDict.__init__(self, d)
        self.botname = self.botname or botname
        self.idnr = i
        self.nick = self.nick or nick
        self.time = ttime
        self.txt = self.txt or txt
        self.printto = self.printto or printto or nick or ""

    def __str__(self):
        result = "%s %s %s %s %s" % (self.botname, self.idnr, self.nick, \
self.time, self.txt)
        return result

class Alarms(Persist):

    """ class that holds the alarms """

    def __init__(self, fname):
        Persist.__init__(self, fname)
        if not self.data:
            self.data = []
        for i in self.data:
            z = Alarmitem(d=i)
            periodical.addjob(z.time - time.time(), 1, self.alarmsay, z.nick, \
z)

    def size(self):
        """ return number of alarms """
        return len(self.data)

    def bynick(self, nick):
        """ get alarms by nick """
        nick = nick.lower()
        result = []
        for i in self.data:
            z = Alarmitem(d=i)
            if z.nick == nick:
                result.append(z)
        return result

    def alarmsay(self, item):
        """ say alarm txt """
        bot = fleet.byname(item.botname)
        if bot:
            if item.printto:
                bot.say(item.printto, "[%s] %s" % (item.nick, item.txt), \
speed=1)
            else:
                bot.say(item.nick, item.txt, speed=1)
        self.delete(item.idnr)

    def add(self, botname, nick, ttime, txt, printto=None):
        """ add alarm """
        nick = nick.lower()
        nrid = nextid.next('alarms')
        item = Alarmitem(botname, nrid, nick, ttime, txt, printto=printto)
        pid = periodical.addjob(ttime - time.time(), 1, self.alarmsay, nick, \
item)
        item.idnr = pid
        self.data.append(item)
        self.save()
        return pid

    def delete(self, idnr):
        """ delete alarmnr """
        for i in range(len(self.data)-1, -1, -1):
            if Alarmitem(d=self.data[i]).idnr == idnr:
                del self.data[i]
                periodical.killjob(idnr)
                self.save()
                return 1

alarms = Alarms(datadir + os.sep + 'plugs' + os.sep + 'alarm' + os.sep + 'alarms')
if not alarms.data:
    alarms = Alarms(datadir + os.sep + 'plugs' + os.sep + 'alarm' + os.sep + 'alarms')
   
def shutdown():
    periodical.kill()

def size():
    """ return number of alarms """
    return alarms.size()

def handle_alarmadd(bot, ievent):
    """ alarm <txt-with-time> | <+delta> <txt> .. add an alarm """
    #if ievent.cmnd == 'DCC':
    #    ievent.reply("sorry can't run alarm from dcc chat")
    #    return 
    if not ievent.rest:
        ievent.reply('alarm <txt-with-time> or alarm <+delta> <txt>')
        return
    else:
        alarmtxt = ievent.rest
    # see if alarm time is given as delta aka starts with +
    if alarmtxt[0] == '+':
        try:
            sec = int(ievent.args[0][1:]) # get nr of seconds to sleep
        except ValueError:
            ievent.reply('use +nrofsecondstosleep')
            return
        if len(ievent.args) < 2:
            ievent.reply('i need txt to remind you')
            return
        try:
            ttime = time.time() + sec
            # check for time overflow
            if ttime > 2**31:
                ievent.reply("time overflow")
                return
            # add alarm 
            if bot.jabber:
                nrid = alarms.add(bot.name, ievent.jid, ttime, \
' '.join(ievent.args[1:]))
            else:
                nrid = alarms.add(bot.name, ievent.nick, ttime, \
' '.join(ievent.args[1:]), ievent.printto)
            ievent.reply("alarm %s set at %s" % (nrid, time.ctime(ttime)))
            return
        except Exception, ex:
            handle_exception(ievent)
            return
    # see if we can determine time from txt
    alarmtime = strtotime(alarmtxt)
    if not alarmtime:
        ievent.reply("can't detect time")
        return
    # check if alarm txt is provided
    txt = striptime(alarmtxt).strip()
    if not txt:
        ievent.reply('i need txt to remind you')
        return
    if time.time() > alarmtime:
        ievent.reply("we are already past %s" % time.ctime(alarmtime))
        return
    # add alarm
    if bot.jabber:
        nrid = alarms.add(bot.name, ievent.jid, alarmtime, txt)
    else:
        nrid = alarms.add(bot.name, ievent.nick, alarmtime, txt, ievent.printto)
    ievent.reply("alarm %s set at %s" % (nrid, time.ctime(alarmtime)))

cmnds.add('alarm', handle_alarmadd, 'USER', allowqueue=False)
examples.add('alarm', 'say txt at a specific time or time diff', \
'1) alarm 12:00 lunchtime 2) alarm 3-11-2008 0:01 birthday ! 3) alarm +180 \
egg ready')
tests.add('alarm 23:59 sleeptime', 'alarm (\d+) set at').add('alarm-del %s')

def handle_alarmlist(bot, ievent):
    """ alarm-list .. list all alarms """
    result = []
    for alarmdata in alarms.data:
        i = Alarmitem(d=alarmdata)
        result.append("%s) %s: %s - %s " % (i.idnr, i.nick, \
time.ctime(i.time), i.txt))
    if result:
        ievent.reply("alarmlist: ", result, dot=True)
    else:
        ievent.reply('no alarms')

cmnds.add('alarm-list', handle_alarmlist, 'OPER')
examples.add('alarm-list', 'list current alarms', 'alarm-list')
tests.add('alarm 23:59 sleeptime', 'alarm (\d+) set at').add('alarm-list', 'sleeptime').add('alarm-del %s')

def handle_myalarmslist(bot, ievent):
    """ alarm-mylist .. show alarms of user giving the command """
    result = []
    if bot.jabber:
        nick = ievent.jid.lower()
    else:
        nick = ievent.nick.lower()
    for alarmdata in alarms.data:
        i = Alarmitem(d=alarmdata)
        if i.nick == nick:
            result.append("%s) %s - %s " % (i.idnr, time.ctime(i.time), i.txt))
    if result:
        ievent.reply("your alarms: ", result, dot=True)
    else:
        ievent.reply('no alarms')

cmnds.add('alarm-mylist', handle_myalarmslist, 'USER')
examples.add('alarm-mylist', 'list alarms of user giving the commands', \
'alarm-mylist')
aliases.data['myalarms'] = 'alarm-mylist'
tests.add('alarm 23:59 sleeptime', 'alarm (\d+) set at').add('alarm-mylist', 'sleeptime').add('alarm-del %s')

def handle_alarmdel(bot, ievent):
    """ alarm-del <nr> .. delete alarm """
    try:
        alarmnr = int(ievent.args[0])
    except IndexError:
        ievent.missing('<nr>')
        return
    except ValueError:
        ievent.reply('argument needs to be an integer')
        return
    if alarms.delete(alarmnr):
        ievent.reply('alarm with id %s deleted' % alarmnr)
    else:
        ievent.reply('failed to delete alarm with id %s' % alarmnr)

cmnds.add('alarm-del', handle_alarmdel, 'OPER')
examples.add('alarm-del', 'delete alarm with id <nr>', 'alarm-del 7')




# plugs/drinks.py
#
#

from gozerbot.commands import cmnds
from gozerbot.examples import examples
from gozerbot.plughelp import plughelp
import os, string, random

plughelp.add('drinks', 'serve coffee/tea or beer')

coffee = ['booo', ]
tea = ['booo', ]
beer = ['booo', ]

def init():
    global coffee
    global tea
    global beer
    for i in  coffeetxt.split('\n'):
        if i:
            coffee.append(i.strip())
    for i in teatxt.split('\n'):
        if i:
            tea.append(i.strip())
    for i in beertxt.split('\n'):
        if i:
            beer.append(i.strip())
    return 1

def handle_coffee(bot, ievent):
    """ get a coffee """
    rand = random.randint(1,len(coffee))
    bot.action(ievent.channel,coffee[rand-1])    

def handle_tea(bot, ievent):
    """ get a tea """
    rand = random.randint(1,len(tea))
    bot.action(ievent.channel,tea[rand-1])
    
def handle_beer(bot, ievent):
    """ get a beer  """
    rand = random.randint(1,len(beer))
    bot.action(ievent.channel,beer[rand-1])

cmnds.add('coffee', handle_coffee, 'USER')
examples.add('coffee', 'get a coffee quote', 'coffee')

cmnds.add('tea', handle_tea, 'USER')
examples.add('tea', 'get an tea', 'tea')

cmnds.add('beer', handle_beer, 'USER')
examples.add('beer', 'get a beer', 'beer')

coffeetxt = """ pours a cup of coffee with two sweets..
pours a cup of espresso for you
gives you a glass of irish coffee
gives you a cappuccino
"""

teatxt = """ tea is for pussies!
"""

beertxt = """ gives you a warsteiner halfom. cheers!
gives a leffe blond. enjoy!
"""



# plugins/autovoice.py
#
#

""" do voice on join """

__copyright__ = 'this file is in the public domain'

from gozerbot.commands import cmnds
from gozerbot.callbacks import callbacks
from gozerbot.examples import examples
from gozerbot.plughelp import plughelp
from gozerbot.tests import tests

plughelp.add('autovoice', 'enable auto voicing of people join .. commands \
work for the channels the commands are given in')

def preautovoice(bot, ievent):
    """ precondition on auto-op .. we must be op """
    if ievent.channel in bot.state['opchan']:
        return 1

def cbautovoice(bot, ievent):
    """ autovoice callback """
    chandata = 0
    try:
        chandata = bot.channels[ievent.channel]['autovoice']
    except KeyError:
        return
    if chandata:
        bot.voice(ievent.channel, ievent.nick)

callbacks.add('JOIN', cbautovoice, preautovoice)

def handle_autovoiceon(bot, ievent):
    """ autovoice-on .. enable autovoice for channel the command was given \
        in """
    try:
        bot.channels[ievent.channel]['autovoice'] = 1
    except TypeError:
        ievent.reply('no %s in channel database' % ievent.channel)
        return
    ievent.reply('autovoice enabled on %s' % ievent.channel)

cmnds.add('autovoice-on', handle_autovoiceon, 'OPER')
examples.add('autovoice-on', 'enable autovoice on channel in which the \
command is given', 'autovoice-on')

def handle_autovoiceoff(bot, ievent):
    """ autovoice-off .. disable autovoice for the channel the command was \
        given in """
    try:
        bot.channels[ievent.channel]['autovoice'] = 0
        ievent.reply('autovoice disabled on %s' % ievent.channel)
    except TypeError:
        ievent.reply('no %s channel in database' % ievent.channel)

cmnds.add('autovoice-off', handle_autovoiceoff, 'OPER')
examples.add('autovoice-off', 'disable autovoice on channel in which \
the command is given', 'autovoice-off')
tests.add('autovoice-on --chan #dunkbots', 'enabled').fakein(':dunker!mekker@127.0.0.1 JOIN #dunkbots').add('autovoice-off --chan #dunkbots', 'disabled')
